@@ -1,16 +1,15 @@
-#!/usr/bin/env python
 ###############################################################################
 # BRLTTY - A background process providing access to the console screen (when in
 #          text mode) for a blind person using a refreshable braille display.
 #
-# Copyright (C) 1995-2017 by The BRLTTY Developers.
+# Copyright (C) 1995-2018 by The BRLTTY Developers.
 #
 # BRLTTY comes with ABSOLUTELY NO WARRANTY.
 #
 # This is free software, placed under the terms of the
-# GNU General Public License, as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any
-# later version. Please see the file LICENSE-GPL for details.
+# GNU Lesser General Public License, as published by the Free Software
+# Foundation; either version 2.1 of the License, or (at your option) any
+# later version. Please see the file LICENSE-LGPL for details.
 #
 # Web Page: http://brltty.com/
 #
@@ -96,6 +95,7 @@ if __name__ == "__main__":
 
     try:
       writeProperty("Driver", brl.driverName)
+      writeProperty("Model", brl.modelIdentifier)
       writeProperty("Columns", str(brl.displaySize[0]))
       writeProperty("Rows", str(brl.displaySize[1]))
 
