@@ -2,7 +2,7 @@
 ###############################################################################
 # libbrlapi - A library providing access to braille terminals for applications.
 #
-# Copyright (C) 2006-2017 by Dave Mielke <dave@mielke.cc>
+# Copyright (C) 2006-2018 by Dave Mielke <dave@mielke.cc>
 #
 # libbrlapi comes with ABSOLUTELY NO WARRANTY.
 #
@@ -52,6 +52,9 @@ puts "FileDescriptor: $fileDescriptor"
 set driverName [$session getDriverName]
 puts "DriverName: $driverName"
 
+set modelIdentifier [$session getModelIdentifier]
+puts "ModelIdentifier: $modelIdentifier"
+
 set displaySize [$session getDisplaySize]
 puts "DisplaySize: [expandList $displaySize width height]"
 
