@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2006-2017 by Dave Mielke <dave@mielke.cc>
+ * Copyright (C) 2006-2018 by Dave Mielke <dave@mielke.cc>
  *
  * libbrlapi comes with ABSOLUTELY NO WARRANTY.
  *
@@ -365,6 +365,7 @@ brlapiSessionCommand (ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj *co
     "getDriverName",
     "getFileDescriptor",
     "getHost",
+    "getModelIdentifier",
     "ignoreKeyRanges",
     "ignoreKeys",
     "leaveRawMode",
@@ -392,6 +393,7 @@ brlapiSessionCommand (ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj *co
     FCN_getDriverName,
     FCN_getFileDescriptor,
     FCN_getHost,
+    FCN_getModelIdentifier,
     FCN_ignoreKeyRanges,
     FCN_ignoreKeys,
     FCN_leaveRawMode,
@@ -476,6 +478,32 @@ brlapiSessionCommand (ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj *co
       }
     }
 
+    case FCN_getModelIdentifier: {
+      size_t size = 0X10;
+
+      if (objc != 2) {
+        Tcl_WrongNumArgs(interp, 2, objv, NULL);
+        return TCL_ERROR;
+      }
+
+      while (1) {
+        char buffer[size];
+        int result = brlapi__getModelIdentifier(session->handle, buffer, size);
+
+        if (result == -1) {
+          setBrlapiError(interp);
+          return TCL_ERROR;
+        }
+
+        if (result <= size) {
+          setStringResult(interp, buffer, result-1);
+          return TCL_OK;
+        }
+
+        size = result;
+      }
+    }
+
     case FCN_getDisplaySize: {
       unsigned int width, height;
 
