@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -18,7 +18,7 @@
 
 /* EcoBraille/braille.c - Braille display library for ECO Braille series
  * Copyright (C) 1999 by Oscar Fernandez <ofa@once.es>
- * See the GNU Public license for details in the LICENSE-GPL file
+ * See the GNU Lesser General Public license for details in the LICENSE-LGPL file
  *
  * For debuging define DEBUG variable
  */
@@ -151,8 +151,8 @@ static int brl_construct(BrailleDisplay *brl, char **parameters, const char *dev
   short ModelID = MODEL;
   unsigned char buffer[DIM_BRL_ID + 6];
 
-  if (!isSerialDevice(&device)) {
-    unsupportedDevice(device);
+  if (!isSerialDeviceIdentifier(&device)) {
+    unsupportedDeviceIdentifier(device);
     return 0;
   }
 
