@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -30,25 +30,11 @@
 #include "brl_driver.h"
 #include "brldefs-hw.h"
 
-BEGIN_KEY_NAME_TABLE(nav)
-  KEY_NAME_ENTRY(HW_KEY_Reset, "Reset"),
-
-  KEY_NAME_ENTRY(HW_KEY_Command1, "Display1"),
-  KEY_NAME_ENTRY(HW_KEY_Command2, "Display2"),
-  KEY_NAME_ENTRY(HW_KEY_Command3, "Display3"),
-  KEY_NAME_ENTRY(HW_KEY_Command4, "Display4"),
-  KEY_NAME_ENTRY(HW_KEY_Command5, "Display5"),
-  KEY_NAME_ENTRY(HW_KEY_Command6, "Display6"),
-
-  KEY_NAME_ENTRY(HW_KEY_Thumb1, "Thumb1"),
-  KEY_NAME_ENTRY(HW_KEY_Thumb2, "Thumb2"),
-  KEY_NAME_ENTRY(HW_KEY_Thumb3, "Thumb3"),
-  KEY_NAME_ENTRY(HW_KEY_Thumb4, "Thumb4"),
-
+BEGIN_KEY_NAME_TABLE(routing)
   KEY_GROUP_ENTRY(HW_GRP_RoutingKeys, "RoutingKey"),
 END_KEY_NAME_TABLE
 
-BEGIN_KEY_NAME_TABLE(kbd)
+BEGIN_KEY_NAME_TABLE(braille)
   KEY_NAME_ENTRY(HW_KEY_Dot1, "Dot1"),
   KEY_NAME_ENTRY(HW_KEY_Dot2, "Dot2"),
   KEY_NAME_ENTRY(HW_KEY_Dot3, "Dot3"),
@@ -60,23 +46,115 @@ BEGIN_KEY_NAME_TABLE(kbd)
   KEY_NAME_ENTRY(HW_KEY_Space, "Space"),
 END_KEY_NAME_TABLE
 
-BEGIN_KEY_NAME_TABLES(mb1)
-  KEY_NAME_TABLE(nav),
+BEGIN_KEY_NAME_TABLE(command)
+  KEY_NAME_ENTRY(HW_KEY_Command1, "Display1"),
+  KEY_NAME_ENTRY(HW_KEY_Command2, "Display2"),
+  KEY_NAME_ENTRY(HW_KEY_Command3, "Display3"),
+  KEY_NAME_ENTRY(HW_KEY_Command4, "Display4"),
+  KEY_NAME_ENTRY(HW_KEY_Command5, "Display5"),
+  KEY_NAME_ENTRY(HW_KEY_Command6, "Display6"),
+END_KEY_NAME_TABLE
+
+BEGIN_KEY_NAME_TABLE(joystick)
+  KEY_NAME_ENTRY(HW_KEY_Up, "Up"),
+  KEY_NAME_ENTRY(HW_KEY_Down, "Down"),
+  KEY_NAME_ENTRY(HW_KEY_Left, "Left"),
+  KEY_NAME_ENTRY(HW_KEY_Right, "Right"),
+  KEY_NAME_ENTRY(HW_KEY_Action, "Action"),
+END_KEY_NAME_TABLE
+
+BEGIN_KEY_NAME_TABLE(thumb)
+  KEY_NAME_ENTRY(HW_KEY_Thumb1, "Previous"),
+  KEY_NAME_ENTRY(HW_KEY_Thumb2, "Back"),
+  KEY_NAME_ENTRY(HW_KEY_Thumb3, "Advance"),
+  KEY_NAME_ENTRY(HW_KEY_Thumb4, "Next"),
+END_KEY_NAME_TABLE
+
+BEGIN_KEY_NAME_TABLES(BI14)
+  KEY_NAME_TABLE(routing),
+  KEY_NAME_TABLE(thumb),
+  KEY_NAME_TABLE(braille),
+  KEY_NAME_TABLE(joystick),
 END_KEY_NAME_TABLES
 
-BEGIN_KEY_NAME_TABLES(mb2)
-  KEY_NAME_TABLE(nav),
-  KEY_NAME_TABLE(kbd),
+BEGIN_KEY_NAME_TABLES(BI32)
+  KEY_NAME_TABLE(routing),
+  KEY_NAME_TABLE(thumb),
+  KEY_NAME_TABLE(braille),
+  KEY_NAME_TABLE(command),
 END_KEY_NAME_TABLES
 
-DEFINE_KEY_TABLE(mb1)
-DEFINE_KEY_TABLE(mb2)
+BEGIN_KEY_NAME_TABLES(BI40)
+  KEY_NAME_TABLE(routing),
+  KEY_NAME_TABLE(thumb),
+  KEY_NAME_TABLE(braille),
+  KEY_NAME_TABLE(command),
+END_KEY_NAME_TABLES
+
+BEGIN_KEY_NAME_TABLES(B80)
+  KEY_NAME_TABLE(routing),
+  KEY_NAME_TABLE(thumb),
+  KEY_NAME_TABLE(command),
+END_KEY_NAME_TABLES
+
+BEGIN_KEY_NAME_TABLES(touch)
+  KEY_NAME_TABLE(routing),
+  KEY_NAME_TABLE(thumb),
+  KEY_NAME_TABLE(braille),
+END_KEY_NAME_TABLES
+
+DEFINE_KEY_TABLE(BI14)
+DEFINE_KEY_TABLE(BI32)
+DEFINE_KEY_TABLE(BI40)
+DEFINE_KEY_TABLE(B80)
+DEFINE_KEY_TABLE(touch)
 
 BEGIN_KEY_TABLE_LIST
-  &KEY_TABLE_DEFINITION(mb1),
-  &KEY_TABLE_DEFINITION(mb2),
+  &KEY_TABLE_DEFINITION(BI14),
+  &KEY_TABLE_DEFINITION(BI32),
+  &KEY_TABLE_DEFINITION(BI40),
+  &KEY_TABLE_DEFINITION(B80),
+  &KEY_TABLE_DEFINITION(touch),
 END_KEY_TABLE_LIST
 
+typedef struct {
+  const KeyTableDefinition *keyTableDefinition;
+  unsigned hasBrailleKeys:1;
+  unsigned hasCommandKeys:1;
+  unsigned hasJoystick:1;
+  unsigned hasSecondThumbKeys:1;
+} ModelEntry;
+
+static const ModelEntry modelEntry_BI14 = {
+  .hasBrailleKeys = 1,
+  .hasJoystick = 1,
+  .keyTableDefinition = &KEY_TABLE_DEFINITION(BI14)
+};
+
+static const ModelEntry modelEntry_BI32 = {
+  .hasBrailleKeys = 1,
+  .hasCommandKeys = 1,
+  .keyTableDefinition = &KEY_TABLE_DEFINITION(BI32)
+};
+
+static const ModelEntry modelEntry_BI40 = {
+  .hasBrailleKeys = 1,
+  .hasCommandKeys = 1,
+  .keyTableDefinition = &KEY_TABLE_DEFINITION(BI40)
+};
+
+static const ModelEntry modelEntry_B80 = {
+  .hasCommandKeys = 1,
+  .hasSecondThumbKeys = 1,
+  .keyTableDefinition = &KEY_TABLE_DEFINITION(B80)
+};
+
+static const ModelEntry modelEntry_touch = {
+  .hasBrailleKeys = 1,
+  .hasCommandKeys = 0,
+  .keyTableDefinition = &KEY_TABLE_DEFINITION(touch)
+};
+
 #define OPEN_READY_DELAY 100
 
 #define SERIAL_PROBE_RESPONSE_TIMEOUT 1000
@@ -93,6 +171,7 @@ END_KEY_TABLE_LIST
 #define BRAILLE_KEY_COUNT (8 + 1)
 #define COMMAND_KEY_COUNT 6
 #define THUMB_KEY_COUNT 4
+#define JOYSTICK_KEY_COUNT 5
 
 typedef struct {
   const char *name;
@@ -104,7 +183,10 @@ typedef struct {
 
 struct BrailleDataStruct {
   const ProtocolEntry *protocol;
+  const ModelEntry *model;
+
   uint32_t firmwareVersion;
+  unsigned isBrailleNoteTouch:1;
   unsigned isOffline:1;
 
   struct {
@@ -130,22 +212,39 @@ struct BrailleDataStruct {
   } hid;
 };
 
+static const ModelEntry *
+getModelEntry (BrailleDisplay *brl) {
+  if (brl->data->isBrailleNoteTouch) return &modelEntry_touch;
+
+  switch (brl->textColumns) {
+    case 14: return &modelEntry_BI14;
+    case 32: return &modelEntry_BI32;
+    case 40: return &modelEntry_BI40;
+    case 80: return &modelEntry_B80;
+    default: return NULL;
+  }
+}
+
+static void
+setModel (BrailleDisplay *brl) {
+  brl->data->model = getModelEntry(brl);
+}
+
 static int
 getDecimalValue (const char *digits, unsigned int count) {
+  const char *end = digits + count;
   unsigned int result = 0;
-
   const char zero = '0';
-  const char nine = '9';
 
-  const char *digit = digits;
-  const char *end = digit + count;
+  while (digits < end) {
+    char digit = *digits++;
+    if (digit >= zero) digit -= zero;
 
-  while (digit < end) {
-    if (*digit < zero) return 0;
-    if (*digit > nine) return 0;
+    if (digit < 0) return 0;
+    if (digit > 9) return 0;
 
     result *= 10;
-    result += *digit++ - zero;
+    result += digit;
   }
 
   return result;
@@ -157,20 +256,6 @@ setFirmwareVersion (BrailleDisplay *brl, unsigned char major, unsigned char mino
   brl->data->firmwareVersion = (major << 16) | (minor << 8) << (build << 0);
 }
 
-static int
-hasBrailleKeyboard (BrailleDisplay *brl) {
-  if (brl->textColumns == 18) return 1;
-  if (brl->textColumns == 32) return 1;
-  if (brl->textColumns == 40) return 1;
-  return 0;
-}
-
-static int
-hasSecondThumbKeys (BrailleDisplay *brl) {
-  if (brl->textColumns == 80) return 1;
-  return 0;
-}
-
 static int
 handleKeyEvent (BrailleDisplay *brl, unsigned char key, int press) {
   KeyGroup group;
@@ -363,7 +448,14 @@ probeSerialDisplay (BrailleDisplay *brl) {
                response.fields.data.init.modelIdentifier,
                response.fields.data.init.cellCount);
 
+    switch (response.fields.data.init.modelIdentifier) {
+      case HW_MODEL_BrailleNoteTouch:
+        brl->data->isBrailleNoteTouch = 1;
+        break;
+    }
+
     brl->textColumns = response.fields.data.init.cellCount;
+    setModel(brl);
 
     writeSerialRequest(brl, HW_MSG_GET_FIRMWARE_VERSION);
     writeSerialRequest(brl, HW_MSG_GET_KEYS);
@@ -540,12 +632,15 @@ probeHidDisplay (BrailleDisplay *brl) {
     getDecimalValue(&capabilities.version.build[0], 2));
 
   brl->textColumns = capabilities.cellCount;
+  setModel(brl);
 
   {
     unsigned char *size = &brl->data->hid.pressedKeys.reportSize;
-    *size = 1 + THUMB_KEY_COUNT + COMMAND_KEY_COUNT + brl->textColumns;
-    if (hasBrailleKeyboard(brl)) *size += BRAILLE_KEY_COUNT;
-    if (hasSecondThumbKeys(brl)) *size += THUMB_KEY_COUNT;
+    *size = 1 + THUMB_KEY_COUNT + brl->textColumns;
+    if (brl->data->model->hasBrailleKeys) *size += BRAILLE_KEY_COUNT;
+    if (brl->data->model->hasCommandKeys) *size += COMMAND_KEY_COUNT;
+    if (brl->data->model->hasJoystick) *size += JOYSTICK_KEY_COUNT;
+    if (brl->data->model->hasSecondThumbKeys) *size += THUMB_KEY_COUNT;
   }
 
   return 1;
@@ -612,6 +707,26 @@ static const ProtocolEntry hidProtocol = {
   .keepAwake = keepHidAwake
 };
 
+typedef struct {
+  const ProtocolEntry *protocol;
+  unsigned isTouch:1;
+} ResourceData;
+
+static const ResourceData resourceData_serial = {
+  .isTouch = 0, // probing detects if it's a Touch or not
+  .protocol = &serialProtocol
+};
+
+static const ResourceData resourceData_HID = {
+  .isTouch = 0, // only for non-Touch models
+  .protocol = &hidProtocol
+};
+
+static const ResourceData resourceData_touch = {
+  .isTouch = 1,
+  .protocol = &hidProtocol
+};
+
 static int
 connectResource (BrailleDisplay *brl, const char *identifier) {
   static const SerialParameters serialParameters = {
@@ -626,21 +741,21 @@ connectResource (BrailleDisplay *brl, const char *identifier) {
       .configuration=1, .interface=1, .alternative=0,
       .inputEndpoint=2, .outputEndpoint=3,
       .serial = &serialParameters,
-      .data = &serialProtocol
+      .data = &resourceData_serial
     },
 
-    { /* all models (HID protocol) */
+    { /* non-Touch models (HID protocol) */
       .vendor=0X1C71, .product=0XC006,
       .configuration=1, .interface=0, .alternative=0,
       .inputEndpoint=1,
-      .data = &hidProtocol
+      .data = &resourceData_HID
     },
 
     { /* BrailleNote Touch (HID protocol) */
       .vendor=0X1C71, .product=0XC00A,
       .configuration=1, .interface=0, .alternative=0,
       .inputEndpoint=1,
-      .data = &hidProtocol
+      .data = &resourceData_touch
     },
   END_USB_CHANNEL_DEFINITIONS
 
@@ -648,18 +763,20 @@ connectResource (BrailleDisplay *brl, const char *identifier) {
   gioInitializeDescriptor(&descriptor);
 
   descriptor.serial.parameters = &serialParameters;
-  descriptor.serial.options.applicationData = &serialProtocol;
+  descriptor.serial.options.applicationData = &resourceData_serial;
   descriptor.serial.options.readyDelay = OPEN_READY_DELAY;
 
   descriptor.usb.channelDefinitions = usbChannelDefinitions;
   descriptor.usb.options.readyDelay = OPEN_READY_DELAY;
 
   descriptor.bluetooth.channelNumber = 1;
-  descriptor.bluetooth.options.applicationData = &serialProtocol;
+  descriptor.bluetooth.options.applicationData = &resourceData_serial;
   descriptor.bluetooth.options.readyDelay = OPEN_READY_DELAY;
 
   if (connectBrailleResource(brl, identifier, &descriptor, NULL)) {
-    brl->data->protocol = gioGetApplicationData(brl->gioEndpoint);
+    const ResourceData *resourceData = gioGetApplicationData(brl->gioEndpoint);
+    brl->data->protocol = resourceData->protocol;
+    brl->data->isBrailleNoteTouch = resourceData->isTouch;
     return 1;
   }
 
@@ -678,15 +795,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 
     if (connectResource(brl, device)) {
       if (brl->data->protocol->probeDisplay(brl)) {
-        {
-          const KeyTableDefinition *ktd =
-            hasBrailleKeyboard(brl)?
-            &KEY_TABLE_DEFINITION(mb2):
-            &KEY_TABLE_DEFINITION(mb1);
-
-          setBrailleKeyTable(brl, ktd);
-        }
-
+        setBrailleKeyTable(brl, brl->data->model->keyTableDefinition);
         makeOutputTable(dotsTable_ISO11548_1);
         brl->data->text.rewrite = 1;
         return 1;
