@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -1848,7 +1848,7 @@ startLatchMonitor (BrailleDisplay *brl) {
   if (brl->data->latch.monitor) return 1;
   if (!brl->data->latch.delay) return 1;
 
-  if (asyncSetAlarmIn(&brl->data->latch.monitor, 0, irMonitorLatch, brl)) {
+  if (asyncNewRelativeAlarm(&brl->data->latch.monitor, 0, irMonitorLatch, brl)) {
     if (asyncResetAlarmEvery(brl->data->latch.monitor, brl->data->latch.interval)) {
       brl->data->latch.pulled = 0;
       return 1;
