@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -282,7 +282,7 @@ ASYNC_ALARM_CALLBACK(handleUsbStatusAlarm) {
 
 static int
 setUsbStatusAlarm (BrailleDisplay *brl) {
-  return asyncSetAlarmIn(&brl->data->proto.usb.statusAlarm,
+  return asyncNewRelativeAlarm(&brl->data->proto.usb.statusAlarm,
                          BRAILLE_DRIVER_INPUT_POLL_INTERVAL,
                          handleUsbStatusAlarm, brl);
 }
