@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -57,7 +57,6 @@
 
 #include "log.h"
 #include "io_misc.h"
-#include "device.h"
 #include "parse.h"
 #include "async_wait.h"
 #include "charset.h"
@@ -958,7 +957,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   inputEnd = 0;
   outputLength = 0;
 
-  if (isQualifiedDevice(&device, "client")) {
+  if (hasQualifier(&device, "client")) {
     static const ModeEntry clientModeEntry = {
 #ifdef AF_LOCAL
       requestLocalConnection,
@@ -971,7 +970,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
       requestInetConnection
     };
     mode = &clientModeEntry;
-  } else if (isQualifiedDevice(&device, "server")) {
+  } else if (hasQualifier(&device, "server")) {
     static const ModeEntry serverModeEntry = {
 #ifdef AF_LOCAL
       acceptLocalConnection,
@@ -985,7 +984,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     };
     mode = &serverModeEntry;
   } else {
-    unsupportedDevice(device);
+    unsupportedDeviceIdentifier(device);
     goto failed;
   }
   if (!*device) device = VR_DEFAULT_SOCKET;
