@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -720,15 +720,19 @@ static int recurseFindTerm(const char *sender, const char *path, int active, int
   dbus_message_iter_recurse (&iter, &iter_array);
   while (dbus_message_iter_get_arg_type (&iter_array) != DBUS_TYPE_INVALID)
   {
-    const char *sender, *path;
+    const char *childsender, *childpath;
     dbus_message_iter_recurse (&iter_array, &iter_struct);
-    dbus_message_iter_get_basic (&iter_struct, &sender);
+    dbus_message_iter_get_basic (&iter_struct, &childsender);
     dbus_message_iter_next (&iter_struct);
-    dbus_message_iter_get_basic (&iter_struct, &path);
-    if (findTerm(sender, path, active, depth))
+    dbus_message_iter_get_basic (&iter_struct, &childpath);
+    /* Make sure that the child is not the same as the parent, to avoid recursing indefinitely.  */
+    if (strcmp(path, childpath))
     {
-      res = 1;
-      goto out;
+      if (findTerm(childsender, childpath, active, depth))
+      {
+	res = 1;
+	goto out;
+      }
     }
     dbus_message_iter_next (&iter_array);
   }
@@ -1089,7 +1093,7 @@ ASYNC_ALARM_CALLBACK(a2ProcessTimeout)
   a2Timeout->monitor = NULL;
   if (dbus_timeout_get_enabled(timeout))
     /* Still enabled, requeue it */
-    asyncSetAlarmIn(&a2Timeout->monitor, dbus_timeout_get_interval(timeout), a2ProcessTimeout, a2Timeout);
+    asyncNewRelativeAlarm(&a2Timeout->monitor, dbus_timeout_get_interval(timeout), a2ProcessTimeout, a2Timeout);
 }
 
 dbus_bool_t a2AddTimeout(DBusTimeout *timeout, void *data)
@@ -1097,7 +1101,7 @@ dbus_bool_t a2AddTimeout(DBusTimeout *timeout, void *data)
   struct a2Timeout *a2Timeout = calloc(1, sizeof(*a2Timeout));
   a2Timeout->timeout = timeout;
   if (dbus_timeout_get_enabled(timeout))
-    asyncSetAlarmIn(&a2Timeout->monitor, dbus_timeout_get_interval(timeout), a2ProcessTimeout, a2Timeout);
+    asyncNewRelativeAlarm(&a2Timeout->monitor, dbus_timeout_get_interval(timeout), a2ProcessTimeout, a2Timeout);
   dbus_timeout_set_data(timeout, a2Timeout, NULL);
   return TRUE;
 }
