@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -85,10 +85,14 @@ extern int isFloat (float *value, const char *string);
 extern int validateFloat (float *value, const char *string, const float *minimum, const float *maximum);
 #endif /* NO_FLOAT */
 
+#define FILE_PATH_DELIMITER  '/'
 #define PARAMETER_SEPARATOR_CHARACTER  ','
 #define PARAMETER_ASSIGNMENT_CHARACTER '='
 #define PARAMETER_QUALIFIER_CHARACTER  ':'
 
+extern int hasQualifier (const char **identifier, const char *qualifier);
+extern int hasNoQualifier (const char *identifier);
+
 extern char **getParameters (const char *const *names, const char *qualifier, const char *parameters);
 extern void logParameters (const char *const *names, char **values, const char *description);
 
