@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2016 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -47,6 +47,33 @@ extern int unlockMutex (pthread_mutex_t *mutex);
 extern size_t formatThreadName (char *buffer, size_t size);
 extern void setThreadName (const char *name);
 
+#if defined(PTHREAD_MUTEX_INITIALIZER)
+typedef pthread_mutex_t CriticalSectionLock;
+#define CRITICAL_SECTION_LOCK_INITIALIZER PTHREAD_MUTEX_INITIALIZER
+
+static inline void
+enterCriticalSection (CriticalSectionLock *lock) {
+  pthread_mutex_lock(lock);
+}
+
+static inline void
+leaveCriticalSection (CriticalSectionLock *lock) {
+  pthread_mutex_unlock(lock);
+}
+
+#else /* critical section lock */
+typedef unsigned char CriticalSectionLock;
+#define CRITICAL_SECTION_LOCK_INITIALIZER 0
+
+static inline void
+enterCriticalSection (CriticalSectionLock *lock) {
+}
+
+static inline void
+leaveCriticalSection (CriticalSectionLock *lock) {
+}
+#endif /* critical section lock */
+
 #define THREAD_SPECIFIC_DATA_NEW(name) void *name##_new (void)
 typedef THREAD_SPECIFIC_DATA_NEW(ThreadSpecificData);
 
