@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -17,7 +17,7 @@
  */
 
 /* apitest provides a small test utility for BRLTTY's API */
- 
+
 #include "prologue.h"
 
 #include <stdio.h>
@@ -34,10 +34,13 @@
 #include "brlapi.h"
 
 static brlapi_connectionSettings_t settings;
+static char *opt_host;
+static char *opt_auth;
 
 static int opt_learnMode;
 static int opt_showDots;
 static int opt_showName;
+static int opt_showModelIdentifier;
 static int opt_showSize;
 static int opt_showKeyCodes;
 static int opt_suspendMode;
@@ -49,6 +52,13 @@ BEGIN_OPTION_TABLE(programOptions)
     .description = "Show the name of the braille driver."
   },
 
+  { .letter = 'm',
+    .word = "model",
+    .setting.flag = &opt_showModelIdentifier,
+    .description = "Show the model identifier of the braille device."
+  },
+
+
   { .letter = 'w',
     .word = "window",
     .setting.flag = &opt_showSize,
@@ -82,14 +92,14 @@ BEGIN_OPTION_TABLE(programOptions)
   { .letter = 'b',
     .word = "brlapi",
     .argument = "[host][:port]",
-    .setting.string = &settings.host,
+    .setting.string = &opt_host,
     .description = "BrlAPIa host and/or port to connect to."
   },
 
   { .letter = 'a',
     .word = "auth",
     .argument = "file",
-    .setting.string = &settings.auth,
+    .setting.string = &opt_auth,
     .description = "BrlAPI authorization/authentication string."
   },
 END_OPTION_TABLE
@@ -116,6 +126,17 @@ static void showDriverName(void)
   fprintf(stderr, "%s\n", name);
 }
 
+static void showModelIdentifier(void)
+{
+  char identifier[30];
+  fprintf(stderr, "Getting model identifier: ");
+  if (brlapi_getModelIdentifier(identifier, sizeof(identifier))<0) {
+    brlapi_perror("failed");
+    exit(PROG_EXIT_FATAL);
+  }
+  fprintf(stderr, "%s\n", identifier);
+}
+
 #define DOTS_TEXT "dots: "
 #define DOTS_TEXTLEN (strlen(DOTS_TEXT))
 #define DOTS_LEN 8
@@ -266,7 +287,6 @@ int
 main (int argc, char *argv[]) {
   ProgramExitStatus exitStatus = PROG_EXIT_SUCCESS;
   brlapi_fileDescriptor fd;
-  settings.host = NULL; settings.auth = NULL;
 
   {
     static const OptionsDescriptor descriptor = {
@@ -276,6 +296,8 @@ main (int argc, char *argv[]) {
     PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
+  settings.host = opt_host;
+  settings.auth = opt_auth;
   fprintf(stderr, "Connecting to BrlAPI... ");
   if ((fd=brlapi_openConnection(&settings, &settings)) != (brlapi_fileDescriptor)(-1)) {
     fprintf(stderr, "done (fd=%"PRIfd")\n", fd);
@@ -285,6 +307,10 @@ main (int argc, char *argv[]) {
       showDriverName();
     }
 
+    if (opt_showModelIdentifier) {
+      showModelIdentifier();
+    }
+
     if (opt_showSize) {
       showDisplaySize();
     }
@@ -306,7 +332,7 @@ main (int argc, char *argv[]) {
     }
 
     brlapi_closeConnection();
-    fprintf(stderr, "Disconnected\n"); 
+    fprintf(stderr, "Disconnected\n");
   } else {
     fprintf(stderr, "failed to connect to %s using auth %s",settings.host, settings.auth);
     brlapi_perror("");
