@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -159,7 +159,7 @@ newAlarmElement (const void *parameters) {
 }
 
 int
-asyncSetAlarmTo (
+asyncNewAbsoluteAlarm (
   AsyncHandle *handle,
   const TimeValue *time,
   AsyncAlarmCallback *callback,
@@ -175,17 +175,17 @@ asyncSetAlarmTo (
 }
 
 int
-asyncSetAlarmIn (
+asyncNewRelativeAlarm (
   AsyncHandle *handle,
-  int interval,
+  int milliseconds,
   AsyncAlarmCallback *callback,
   void *data
 ) {
   TimeValue time;
 
   getMonotonicTime(&time);
-  adjustTimeValue(&time, interval);
-  return asyncSetAlarmTo(handle, &time, callback, data);
+  adjustTimeValue(&time, milliseconds);
+  return asyncNewAbsoluteAlarm(handle, &time, callback, data);
 }
 
 static Element *
@@ -209,22 +209,22 @@ asyncResetAlarmTo (AsyncHandle handle, const TimeValue *time) {
 }
 
 int
-asyncResetAlarmIn (AsyncHandle handle, int interval) {
+asyncResetAlarmIn (AsyncHandle handle, int milliseconds) {
   TimeValue time;
 
   getMonotonicTime(&time);
-  adjustTimeValue(&time, interval);
+  adjustTimeValue(&time, milliseconds);
   return asyncResetAlarmTo(handle, &time);
 }
 
 int
-asyncResetAlarmEvery (AsyncHandle handle, int interval) {
+asyncResetAlarmEvery (AsyncHandle handle, int milliseconds) {
   Element *element = getAlarmElement(handle);
 
   if (element) {
     AlarmEntry *alarm = getElementItem(element);
 
-    alarm->interval = interval;
+    alarm->interval = milliseconds;
     alarm->reschedule = 1;
     return 1;
   }
