@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -33,6 +33,7 @@ BLUETOOTH_NAME_DRIVERS(BaumSuperVario, "bm");
 BLUETOOTH_NAME_DRIVERS(BaumSVario, "bm");
 BLUETOOTH_NAME_DRIVERS(BrailleConnect, "bm");
 BLUETOOTH_NAME_DRIVERS(BrailleEdge, "hm");
+BLUETOOTH_NAME_DRIVERS(BrailleMe, "ic");
 BLUETOOTH_NAME_DRIVERS(BrailleMemoPocket, "mm");
 BLUETOOTH_NAME_DRIVERS(BrailleMemoSmart, "mm");
 BLUETOOTH_NAME_DRIVERS(BrailleMemo32, "mm");
@@ -46,12 +47,14 @@ BLUETOOTH_NAME_DRIVERS(Braillino, "ht");
 BLUETOOTH_NAME_DRIVERS(B2G, "bm");
 BLUETOOTH_NAME_DRIVERS(Conny, "bm");
 BLUETOOTH_NAME_DRIVERS(EL12, "al", "vo");
+BLUETOOTH_NAME_DRIVERS(Eurobraille, "eu");
 BLUETOOTH_NAME_DRIVERS(Focus, "fs");
 BLUETOOTH_NAME_DRIVERS(HWGBrailliant, "bm");
 BLUETOOTH_NAME_DRIVERS(MB248, "md");
 BLUETOOTH_NAME_DRIVERS(OrbitReader, "bm");
 BLUETOOTH_NAME_DRIVERS(Pronto, "bm");
 BLUETOOTH_NAME_DRIVERS(Refreshabraille, "bm");
+BLUETOOTH_NAME_DRIVERS(SmartBeetle, "hm");
 BLUETOOTH_NAME_DRIVERS(SuperVario, "bm");
 BLUETOOTH_NAME_DRIVERS(TSM, "sk");
 BLUETOOTH_NAME_DRIVERS(VarioConnect, "bm");
@@ -106,6 +109,10 @@ const BluetoothNameEntry bluetoothNameTable[] = {
     .driverCodes = bluetoothNameDrivers_BrailleEdge
   },
 
+  { .namePrefix = "BrailleMe",
+    .driverCodes = bluetoothNameDrivers_BrailleMe
+  },
+
   { .namePrefix = "BMpk",
     .driverCodes = bluetoothNameDrivers_BrailleMemoPocket
   },
@@ -158,6 +165,10 @@ const BluetoothNameEntry bluetoothNameTable[] = {
     .driverCodes = bluetoothNameDrivers_EL12
   },
 
+  { .namePrefix = "Esys-",
+    .driverCodes = bluetoothNameDrivers_Eurobraille
+  },
+
   { .namePrefix = "Focus",
     .driverCodes = bluetoothNameDrivers_Focus
   },
@@ -182,6 +193,10 @@ const BluetoothNameEntry bluetoothNameTable[] = {
     .driverCodes = bluetoothNameDrivers_Refreshabraille
   },
 
+  { .namePrefix = "SmartBeetle",
+    .driverCodes = bluetoothNameDrivers_SmartBeetle
+  },
+
   { .namePrefix = "SuperVario",
     .driverCodes = bluetoothNameDrivers_SuperVario
   },
