@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2002-2017 by
+ * Copyright (C) 2002-2018 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *
@@ -76,6 +76,7 @@
 #endif /* HAVE_ALLOCA_H */
 
 #ifdef linux
+#include <sys/sysmacros.h>
 #include <linux/major.h>
 #include <linux/tty.h>
 #include <linux/vt.h>
@@ -126,9 +127,9 @@ static void* GetProc(const char *library, const char *fun) {
 }
 
 #define CHECKPROC(library, name) \
-  	(name##Proc && name##Proc != (void*)(-1))
+  (name##Proc && name##Proc != (void*)(-1))
 #define CHECKGETPROC(library, name) \
-  	(name##Proc != (void*)(-1) && (name##Proc || (name##Proc = GetProc(library,#name)) != (void*)(-1)))
+  (name##Proc != (void*)(-1) && (name##Proc || (name##Proc = GetProc(library,#name)) != (void*)(-1)))
 #define WIN_PROC_STUB(name) typeof(name) (*name##Proc);
 
 
@@ -144,8 +145,9 @@ static WIN_PROC_STUB(freeaddrinfo);
 #endif /* __MINGW32__ */
 #endif /* WINDOWS */
 
-/* We need to declare these as weak external references to determine at runtime 
- * whether libpthread is used or not. We also can't rely on the functions prototypes.
+/* We need to declare these as weak external references to determine
+ * at runtime whether libpthread is used or not.
+ * We also can't rely on the functions prototypes.
  */
 #if defined(WINDOWS)
 
@@ -224,6 +226,11 @@ struct brlapi_handle_t { /* Connection-specific information */
   sem_t *altSem;
   int state;
   pthread_mutex_t state_mutex;
+
+#ifdef LC_GLOBAL_LOCALE
+  locale_t default_locale;
+#endif /* LC_GLOBAL_LOCALE */
+
   /* key presses buffer, for when key presses are received instead of
    * acknowledgements for instance
    *
@@ -266,6 +273,11 @@ static void brlapi_initializeHandle(brlapi_handle_t *handle)
   handle->altSem = NULL;
   handle->state = 0;
   pthread_mutex_init(&handle->state_mutex, NULL);
+
+#ifdef LC_GLOBAL_LOCALE
+  handle->default_locale = LC_GLOBAL_LOCALE;
+#endif /* LC_GLOBAL_LOCALE */
+
   memset(handle->keybuf, 0, sizeof(handle->keybuf));
   handle->keybuf_next = 0;
   handle->keybuf_nb = 0;
@@ -376,7 +388,7 @@ again:
   } else {
     if (
 #ifndef WINDOWS
-    	!sem_init || !sem_post || !sem_wait || !sem_destroy ||
+      !sem_init || !sem_post || !sem_wait || !sem_destroy ||
 #endif /* WINDOWS */
 	handle->altSem) {
       /* This can't happen without threads */
@@ -582,7 +594,7 @@ static int tryHost(brlapi_handle_t *handle, char *hostAndPort) {
       addr.sin_port = htons(strtol(port, &c, 0));
       if (*c) {
 	struct servent *se;
-	
+
 	if (!(se = getservbyname(port,"tcp"))) {
 	  brlapi_gaierrno = h_errno;
           brlapi_errno=BRLAPI_ERROR_GAIERR;
@@ -616,7 +628,7 @@ static int tryHost(brlapi_handle_t *handle, char *hostAndPort) {
       }
       memcpy(&addr.sin_addr,he->h_addr,he->h_length);
     }
-    
+
     sockfd = socket(addr.sin_family, SOCK_STREAM, 0);
     if (sockfd<0) {
       brlapi_errfun = "socket";
@@ -780,6 +792,13 @@ void BRLAPI_STDCALL brlapi__closeConnection(brlapi_handle_t *handle)
   closeFileDescriptor(handle->fileDescriptor);
   handle->fileDescriptor = INVALID_FILE_DESCRIPTOR;
   pthread_mutex_unlock(&handle->fileDescriptor_mutex);
+
+#ifdef LC_GLOBAL_LOCALE
+  if (handle->default_locale != LC_GLOBAL_LOCALE) {
+    freelocale(handle->default_locale);
+  }
+#endif /* LC_GLOBAL_LOCALE */
+
 #ifdef __MINGW32__
   WSACleanup();
 #endif /* __MINGW32__ */
@@ -955,6 +974,20 @@ int BRLAPI_STDCALL brlapi_getDriverName(char *name, size_t n)
   return brlapi__getDriverName(&defaultHandle, name, n);
 }
 
+/* Function : brlapi_getModelIdentifier */
+/* An identifier for the model of the device used by brltty */
+int BRLAPI_STDCALL brlapi__getModelIdentifier(brlapi_handle_t *handle, char *identifier, size_t n)
+{
+  ssize_t res = brlapi__request(handle, BRLAPI_PACKET_GETMODELID, identifier, n);
+  if ((res>0) && (res<=n)) identifier[res-1] = '\0';
+  return res;
+}
+
+int BRLAPI_STDCALL brlapi_getModelIdentifier(char *identifier, size_t n)
+{
+  return brlapi__getModelIdentifier(&defaultHandle, identifier, n);
+}
+
 /* Function : brlapi_getDisplaySize */
 /* Returns the size of the braille display */
 int BRLAPI_STDCALL brlapi__getDisplaySize(brlapi_handle_t *handle, unsigned int *x, unsigned int *y)
@@ -1035,7 +1068,7 @@ int BRLAPI_STDCALL brlapi__enterTtyMode(brlapi_handle_t *handle, int tty, const
   /* 0 can be a valid screen WINDOW
   0xffffffff can not be a valid WINDOWID (top 3 bits guaranteed to be zero) */
   if (tty<0) { brlapi_errno=BRLAPI_ERROR_UNKNOWNTTY; return -1; }
-  
+
   if (brlapi__enterTtyModeWithPath(handle, &tty, 1, driverName)) return -1;
 
   return tty;
@@ -1066,7 +1099,7 @@ int BRLAPI_STDCALL brlapi__enterTtyModeWithPath(brlapi_handle_t *handle, int *tt
   }
 
   if (brlapi__getDisplaySize(handle, &handle->brlx, &handle->brly)<0) return -1;
-  
+
   /* Clear key buffer before taking the tty, just in case... */
   pthread_mutex_lock(&handle->read_mutex);
   handle->keybuf_next = handle->keybuf_nb = 0;
@@ -1099,9 +1132,24 @@ int BRLAPI_STDCALL brlapi__enterTtyModeWithPath(brlapi_handle_t *handle, int *tt
   *p = n;
   p++;
   if (n) p = mempcpy(p, driverName, n);
-  if ((res=brlapi__writePacketWaitForAck(handle,BRLAPI_PACKET_ENTERTTYMODE,&packet,(p-(unsigned char *)&packet))) == 0)
+
+  if ((res=brlapi__writePacketWaitForAck(handle,BRLAPI_PACKET_ENTERTTYMODE,&packet,(p-(unsigned char *)&packet))) == 0) {
     handle->state |= STCONTROLLINGTTY;
+  }
+
   pthread_mutex_unlock(&handle->state_mutex);
+
+  /* Determine default charset if application did not call setlocale.  */
+#ifdef LC_GLOBAL_LOCALE
+  const char *locale = setlocale(LC_CTYPE, NULL);
+
+  if (!locale || !strcmp(locale, "C")) {
+    /* Application did not call setlocale, try to load the current locale.  */
+    locale_t default_locale = newlocale(LC_CTYPE_MASK, "", 0);
+    if (default_locale) handle->default_locale = default_locale;
+  }
+#endif /* LC_GLOBAL_LOCALE */
+
   return res;
 }
 
@@ -1152,7 +1200,7 @@ int BRLAPI_STDCALL brlapi_setFocus(int tty)
   return brlapi__setFocus(&defaultHandle, tty);
 }
 
-static size_t getCharset(void *buffer, int wide) {
+static size_t getCharset(brlapi_handle_t *handle, void *buffer, int wide) {
   char *p = buffer;
   const char *start = p;
   const char *locale = setlocale(LC_CTYPE, NULL);
@@ -1165,7 +1213,11 @@ static size_t getCharset(void *buffer, int wide) {
     size_t length = strlen(WCHAR_CHARSET);
     *p++ = length;
     p = mempcpy(p, WCHAR_CHARSET, length);
-  } else if (locale && strcmp(locale, "C")) {
+  } else if ((locale && strcmp(locale, "C"))
+#ifdef LC_GLOBAL_LOCALE
+             || (handle->default_locale != LC_GLOBAL_LOCALE)
+#endif /* LC_GLOBAL_LOCALE */
+            ) {
     /* not default locale, tell charset to server */
 #ifdef WINDOWS
     UINT CP = GetACP();
@@ -1200,7 +1252,20 @@ static int brlapi___writeText(brlapi_handle_t *handle, int cursor, const void *s
   char *locale;
   int res;
   size_t len;
+
+#ifdef LC_GLOBAL_LOCALE
+  locale_t old_locale = 0;
+
+  if (handle->default_locale != LC_GLOBAL_LOCALE) {
+    /* Temporarily load the default locale.  */
+    old_locale = uselocale(handle->default_locale);
+  }
+
+  locale = "default locale";
+#else /* LC_GLOBAL_LOCALE */
   locale = setlocale(LC_CTYPE,NULL);
+#endif /* LC_GLOBAL_LOCALE */
+
   wa->flags = BRLAPI_WF_REGION;
   *((uint32_t *) p) = htonl(1); p += sizeof(uint32_t);
   *((uint32_t *) p) = htonl(dispSize); p += sizeof(uint32_t);
@@ -1235,6 +1300,12 @@ static int brlapi___writeText(brlapi_handle_t *handle, int cursor, const void *s
 	    brlapi_libcerrno = errno;
 	    brlapi_errfun = "mbrlen";
 	    brlapi_errno = BRLAPI_ERROR_LIBCERR;
+#ifdef LC_GLOBAL_LOCALE
+	    if (handle->default_locale != LC_GLOBAL_LOCALE) {
+	      /* Restore application locale */
+	      uselocale(old_locale);
+            }
+#endif /* LC_GLOBAL_LOCALE */
 	    return -1;
 	  case 0:
 	    goto endcount;
@@ -1269,7 +1340,7 @@ endcount:
     p += sizeof(uint32_t);
   }
 
-  if ((len = getCharset(p , wide))) {
+  if ((len = getCharset(handle, p , wide))) {
     wa->flags |= BRLAPI_WF_CHARSET;
     p += len;
   }
@@ -1278,6 +1349,14 @@ endcount:
   pthread_mutex_lock(&handle->fileDescriptor_mutex);
   res = brlapi_writePacket(handle->fileDescriptor,BRLAPI_PACKET_WRITE,&packet,sizeof(wa->flags)+(p-&wa->data));
   pthread_mutex_unlock(&handle->fileDescriptor_mutex);
+
+#ifdef LC_GLOBAL_LOCALE
+  if (handle->default_locale != LC_GLOBAL_LOCALE) {
+    /* Restore application locale */
+    uselocale(old_locale);
+  }
+#endif /* LC_GLOBAL_LOCALE */
+
   return res;
 }
 
@@ -1429,14 +1508,31 @@ int brlapi__write(brlapi_handle_t *handle, const brlapi_writeArguments_t *s)
     p += sizeof(uint32_t);
   } else if (s->cursor!=-1) {
     brlapi_errno = BRLAPI_ERROR_INVALID_PARAMETER;
-    return -1;    
+    return -1;
   }
+
   if (s->charset) {
     if (!*s->charset) {
-      if ((strLen = getCharset(p, wide))) {
+#ifdef LC_GLOBAL_LOCALE
+      locale_t old_locale = 0;
+
+      if (handle->default_locale != LC_GLOBAL_LOCALE) {
+	/* Temporarily load the default locale.  */
+	old_locale = uselocale(handle->default_locale);
+      }
+#endif /* LC_GLOBAL_LOCALE */
+
+      if ((strLen = getCharset(handle, p, wide))) {
 	wa->flags |= BRLAPI_WF_CHARSET;
 	p += strLen;
       }
+
+#ifdef LC_GLOBAL_LOCALE
+      if (handle->default_locale != LC_GLOBAL_LOCALE) {
+	/* Restore application locale */
+	uselocale(old_locale);
+      }
+#endif /* LC_GLOBAL_LOCALE */
     } else {
       strLen = strlen(s->charset);
       *p++ = strLen;
@@ -1445,10 +1541,12 @@ int brlapi__write(brlapi_handle_t *handle, const brlapi_writeArguments_t *s)
 	brlapi_errno = BRLAPI_ERROR_INVALID_PARAMETER;
 	return -1;
       }
+
       memcpy(p, s->charset, strLen);
       p += strLen;
     }
   }
+
 send:
   wa->flags = htonl(wa->flags);
   pthread_mutex_lock(&handle->fileDescriptor_mutex);
@@ -1702,7 +1800,7 @@ done:
 /* Function : ignore_accept_key_range */
 /* Common tasks for ignoring and unignoring key ranges */
 /* what = 0 for ignoring !0 for unignoring */
-static int ignore_accept_key_ranges(brlapi_handle_t *handle, int what, brlapi_range_t ranges[], unsigned int n)
+static int ignore_accept_key_ranges(brlapi_handle_t *handle, int what, const brlapi_range_t ranges[], unsigned int n)
 {
   uint32_t ints[n][4];
   unsigned int i, remaining, todo;
@@ -1756,12 +1854,12 @@ static int ignore_accept_keys(brlapi_handle_t *handle, int what, brlapi_rangeTyp
 }
 
 /* Function : brlapi_acceptKeyRanges */
-int BRLAPI_STDCALL brlapi__acceptKeyRanges(brlapi_handle_t *handle, brlapi_range_t ranges[], unsigned int n)
+int BRLAPI_STDCALL brlapi__acceptKeyRanges(brlapi_handle_t *handle, const brlapi_range_t ranges[], unsigned int n)
 {
   return ignore_accept_key_ranges(handle, !0, ranges, n);
 }
 
-int BRLAPI_STDCALL brlapi_acceptKeyRanges(brlapi_range_t ranges[], unsigned int n)
+int BRLAPI_STDCALL brlapi_acceptKeyRanges(const brlapi_range_t ranges[], unsigned int n)
 {
   return brlapi__acceptKeyRanges(&defaultHandle, ranges, n);
 }
@@ -1778,12 +1876,12 @@ int BRLAPI_STDCALL brlapi_acceptKeys(brlapi_rangeType_t r, const brlapi_keyCode_
 }
 
 /* Function : brlapi_ignoreKeyRanges */
-int BRLAPI_STDCALL brlapi__ignoreKeyRanges(brlapi_handle_t *handle, brlapi_range_t ranges[], unsigned int n)
+int BRLAPI_STDCALL brlapi__ignoreKeyRanges(brlapi_handle_t *handle, const brlapi_range_t ranges[], unsigned int n)
 {
   return ignore_accept_key_ranges(handle, 0, ranges, n);
 }
 
-int BRLAPI_STDCALL brlapi_ignoreKeyRanges(brlapi_range_t ranges[], unsigned int n)
+int BRLAPI_STDCALL brlapi_ignoreKeyRanges(const brlapi_range_t ranges[], unsigned int n)
 {
   return brlapi__ignoreKeyRanges(&defaultHandle, ranges, n);
 }
