@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -29,7 +29,7 @@
 #endif /* PTHREAD_STACK_MIN */
 
 #define RELEASE "BrlAPI Server: release " BRLAPI_RELEASE
-#define COPYRIGHT "   Copyright (C) 2002-2017 by Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>, \
+#define COPYRIGHT "   Copyright (C) 2002-2018 by Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>, \
 Samuel Thibault <samuel.thibault@ens-lyon.org>"
 
 #include "prologue.h"
@@ -155,7 +155,7 @@ static brlapi_error_t brlapiserver_error;
 
 #define BRLAPI(fun) brlapiserver_ ## fun
 #include "brlapi_common.h"
- 
+
 /** ask for \e brltty commands */
 #define BRL_COMMANDS 0
 /** ask for raw driver keycodes */
@@ -192,7 +192,7 @@ typedef struct {
   PacketState state;
   int readBytes; /* Already read bytes */
   unsigned char *p; /* Where read() should load datas */
-  int n; /* Value to give so read() */ 
+  int n; /* Value to give so read() */
 #ifdef __MINGW32__
   OVERLAPPED overl;
 #endif /* __MINGW32__ */
@@ -538,6 +538,7 @@ typedef int(*PacketHandler)(Connection *, brlapi_packetType_t, brlapi_packet_t *
 
 typedef struct { /* packet handlers */
   PacketHandler getDriverName;
+  PacketHandler getModelIdentifier;
   PacketHandler getDisplaySize;
   PacketHandler enterTtyMode;
   PacketHandler setFocus;
@@ -545,7 +546,7 @@ typedef struct { /* packet handlers */
   PacketHandler ignoreKeyRanges;
   PacketHandler acceptKeyRanges;
   PacketHandler write;
-  PacketHandler enterRawMode;  
+  PacketHandler enterRawMode;
   PacketHandler leaveRawMode;
   PacketHandler packet;
   PacketHandler suspendDriver;
@@ -757,7 +758,7 @@ static inline Tty *newTty(Tty *father, int number)
     tty->next->prevnext = &tty->next;
   father->subttys = tty;
   return tty;
-  
+
 outtty:
   free(tty);
 out:
@@ -806,6 +807,11 @@ static int handleGetDriverName(Connection *c, brlapi_packetType_t type, brlapi_p
   return handleGetDriver(c, type, size, braille->definition.name);
 }
 
+static int handleGetModelIdentifier(Connection *c, brlapi_packetType_t type, brlapi_packet_t *packet, size_t size)
+{
+  return handleGetDriver(c, type, size, disp->keyBindings);
+}
+
 static int handleGetDisplaySize(Connection *c, brlapi_packetType_t type, brlapi_packet_t *packet, size_t size)
 {
   CHECKERR(size==0,BRLAPI_ERROR_INVALID_PACKET,"packet should be empty");
@@ -893,7 +899,7 @@ static int handleEnterTtyMode(Connection *c, brlapi_packetType_t type, brlapi_pa
         unlockMutex(&apiConnectionsMutex);
         WERR(c->fd,BRLAPI_ERROR_NOMEM, "no memory for new tty");
         freeBrailleWindow(&c->brailleWindow);
-  	return 0;
+        return 0;
       }
       logMessage(LOG_CATEGORY(SERVER_EVENTS), "allocated tty %#010lx",(unsigned long)ntohl(*ptty));
     }
@@ -945,7 +951,7 @@ static int handleSetFocus(Connection *c, brlapi_packetType_t type, brlapi_packet
 static void doLeaveTty(Connection *c)
 {
   Tty *tty = c->tty;
-  logMessage(LOG_CATEGORY(SERVER_EVENTS), "fd %"PRIfd"releasing tty %#010x",c->fd,tty->number);
+  logMessage(LOG_CATEGORY(SERVER_EVENTS), "fd %"PRIfd" releasing tty %#010x",c->fd,tty->number);
   c->tty = NULL;
   lockMutex(&apiConnectionsMutex);
   __removeConnection(c);
@@ -1106,6 +1112,7 @@ static int handleWrite(Connection *c, brlapi_packetType_t type, brlapi_packet_t
       if (coreCharset) unlockCharset();
       lockMutex(&c->brailleWindowMutex);
       memcpy(c->brailleWindow.text+rbeg-1,textBuf,rsiz*sizeof(wchar_t));
+      logMessage(LOG_CATEGORY(SERVER_EVENTS), "fd %"PRIfd" wrote %d characters %d bytes",c->fd,rsiz,textLen);
     } else
 #endif /* HAVE_ICONV_H */
     {
@@ -1115,6 +1122,7 @@ static int handleWrite(Connection *c, brlapi_packetType_t type, brlapi_packet_t
 	/* assume latin1 */
         c->brailleWindow.text[rbeg-1+i] = text[i];
       }
+      logMessage(LOG_CATEGORY(SERVER_EVENTS), "fd %"PRIfd" wrote %d characters %d bytes",c->fd,rsiz,rsiz);
     }
     if (!andAttr) memset(c->brailleWindow.andAttr+rbeg-1,0xFF,rsiz);
     if (!orAttr)  memset(c->brailleWindow.orAttr+rbeg-1,0x00,rsiz);
@@ -1224,7 +1232,7 @@ static int handleResumeDriver(Connection *c, brlapi_packetType_t type, brlapi_pa
 }
 
 static PacketHandlers packetHandlers = {
-  handleGetDriverName, handleGetDisplaySize,
+  handleGetDriverName, handleGetModelIdentifier, handleGetDisplaySize,
   handleEnterTtyMode, handleSetFocus, handleLeaveTtyMode,
   handleKeyRanges, handleKeyRanges, handleWrite,
   handleEnterRawMode, handleLeaveRawMode, handlePacket,
@@ -1404,15 +1412,16 @@ static int processRequest(Connection *c, PacketHandlers *handlers)
   }
   size = c->packet.header.size;
   type = c->packet.header.type;
-  
+
   if (c->auth!=1) return handleUnauthorizedConnection(c, type, packet, size);
 
   if (size>BRLAPI_MAXPACKETSIZE) {
     logMessage(LOG_WARNING, "Discarding too large packet of type %s on fd %"PRIfd,brlapiserver_getPacketTypeName(type), c->fd);
-    return 0;    
+    return 0;
   }
   switch (type) {
     case BRLAPI_PACKET_GETDRIVERNAME: p = handlers->getDriverName; break;
+    case BRLAPI_PACKET_GETMODELID: p = handlers->getModelIdentifier; break;
     case BRLAPI_PACKET_GETDISPLAYSIZE: p = handlers->getDisplaySize; break;
     case BRLAPI_PACKET_ENTERTTYMODE: p = handlers->enterTtyMode; break;
     case BRLAPI_PACKET_SETFOCUS: p = handlers->setFocus; break;
@@ -1441,7 +1450,7 @@ static int processRequest(Connection *c, PacketHandlers *handlers)
  * There is one server thread which first launches binding threads and then
  * enters infinite loop trying to accept connections, read packets, etc.
  *
- * Binding threads loop trying to establish some socket, waiting for 
+ * Binding threads loop trying to establish some socket, waiting for
  * filesystems to be read/write or network to be configured.
  *
  * On windows, WSAEventSelect() is emulated by a standalone thread.
@@ -1859,7 +1868,7 @@ static FileDescriptor createLocalSocket(struct socketInfo *info)
   tmppath[lpath+2+lport+1]=0;
   lockpath[lpath+2+lport]=0;
 
-  while ((lock = open(tmppath, O_WRONLY|O_CREAT|O_EXCL, 
+  while ((lock = open(tmppath, O_WRONLY|O_CREAT|O_EXCL,
                       (permissions & (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)))) == -1) {
     if (errno == EROFS) {
       approximateDelay(1000);
@@ -2002,7 +2011,7 @@ static void closeSockets(void *arg)
 {
   int i;
   struct socketInfo *info;
-  
+
   for (i=0;i<serverSocketCount;i++) {
 #ifdef __MINGW32__
     pthread_cancel(socketThreads[i]);
@@ -2291,7 +2300,7 @@ THREAD_FUNCTION(runServer) {
     } else {
       /* Windows doesn't have trouble with local sockets on read-only
        * filesystems, but it has with inter-thread overlapped operations,
-       * so call from here 
+       * so call from here
        */
       createSocket(i);
     }
@@ -2338,16 +2347,6 @@ THREAD_FUNCTION(runServer) {
     FD_ZERO(&sockset);
     fdmax=0;
 
-    for (i=0;i<serverSocketCount;i++) {
-      if (socketInfo[i].fd>=0) {
-	FD_SET(socketInfo[i].fd, &sockset);
-
-	if (socketInfo[i].fd>fdmax) {
-	  fdmax = socketInfo[i].fd;
-        }
-      }
-    }
-
     lockMutex(&apiConnectionsMutex);
     addTtyFds(&sockset, &fdmax, &notty);
     addTtyFds(&sockset, &fdmax, &ttys);
@@ -2357,6 +2356,16 @@ THREAD_FUNCTION(runServer) {
       struct timeval tv, *timeout;
 
       lockMutex(&serverSocketsMutex);
+	for (i=0;i<serverSocketCount;i++) {
+	  if (socketInfo[i].fd>=0) {
+	    FD_SET(socketInfo[i].fd, &sockset);
+
+	    if (socketInfo[i].fd>fdmax) {
+	      fdmax = socketInfo[i].fd;
+	    }
+	  }
+	}
+
         if (unauthConnections || serverSocketsPending) {
           memset(&tv, 0, sizeof(tv));
           tv.tv_sec = SERVER_SELECT_TIMEOUT;
@@ -2476,7 +2485,9 @@ finished:
 /* to let the user read the screen in case theree is an error */
 static int initializeAcceptedKeys(Connection *c, int how)
 {
-  if (how != BRL_KEYCODES) {
+  if (how == BRL_KEYCODES) {
+    if (c && addKeyrange(0, BRLAPI_KEY_MAX, &c->acceptedKeys) == -1) return -1;
+  } else {
     if (c) {
       typedef struct {
         int (*action) (brlapi_keyCode_t first, brlapi_keyCode_t last, KeyrangeList **list);
@@ -3197,7 +3208,7 @@ int api_start(BrailleDisplay *brl, char **parameters)
   }
 
   return 1;
-  
+
 noServerThread:
   asyncDiscardEvent(flushEvent);
 noFlushEvent:
