@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -154,18 +154,22 @@ putCharacter (unsigned char character, void *data) {
 }
 
 static int
-putMappedCharacter (unsigned char cell, void *data) {
-  fputc(convertDotsToCharacter(textTable, cell), outputStream);
+putCellCharacter (wchar_t character, void *data) {
+  Utf8Buffer utf8;
+  size_t utfs = convertWcharToUtf8(character, utf8);
+
+  fprintf(outputStream, "%.*s", (int)utfs, utf8);
   return checkOutputStream(data);
 }
 
 static int
-putUnicodeBraille (unsigned char cell, void *data) {
-  Utf8Buffer utf8;
-  size_t utfs = convertWcharToUtf8(cell|UNICODE_BRAILLE_ROW, utf8);
+putTextCell (unsigned char cell, void *data) {
+  return putCellCharacter(convertDotsToCharacter(textTable, cell), data);
+}
 
-  fprintf(outputStream, "%.*s", (int)utfs, utf8);
-  return checkOutputStream(data);
+static int
+putBrailleCell (unsigned char cell, void *data) {
+  return putCellCharacter((UNICODE_BRAILLE_ROW | cell), data);
 }
 
 static int
@@ -472,7 +476,7 @@ main (int argc, char *argv[]) {
         if (*opt_textTable) {
           char *textTablePath;
 
-          putCell = putMappedCharacter;
+          putCell = putTextCell;
 
           if ((textTablePath = makeTextTablePath(opt_tablesDirectory, opt_textTable))) {
             exitStatus = (textTable = compileTextTable(textTablePath))? PROG_EXIT_SUCCESS: PROG_EXIT_FATAL;
@@ -481,7 +485,7 @@ main (int argc, char *argv[]) {
             exitStatus = PROG_EXIT_FATAL;
           }
         } else {
-          putCell = putUnicodeBraille;
+          putCell = putBrailleCell;
           exitStatus = PROG_EXIT_SUCCESS;
         }
 
