@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -161,7 +161,7 @@ static int
 testIndent (int column, int row, void *data UNUSED) {
   int count = column+1;
   ScreenCharacter characters[count];
-  readScreen(0, row, count, 1, characters);
+  readScreenRow(row, count, characters);
   while (column >= 0) {
     wchar_t text = characters[column].text;
     if (text != WC_C(' ')) return 1;
@@ -175,7 +175,7 @@ testPrompt (int column, int row, void *data) {
   const ScreenCharacter *prompt = data;
   int count = column+1;
   ScreenCharacter characters[count];
-  readScreen(0, row, count, 1, characters);
+  readScreenRow(row, count, characters);
   return isSameRow(characters, prompt, count, isSameText);
 }
 
@@ -357,7 +357,7 @@ handleNavigationCommands (int command, void *data) {
       while (1) {
         int isBlankLine;
 
-        readScreen(0, line, scr.cols, 1, characters);
+        readScreenRow(line, scr.cols, characters);
         isBlankLine = isAllSpaceCharacters(characters, scr.cols);
 
         switch (state) {
@@ -401,7 +401,7 @@ handleNavigationCommands (int command, void *data) {
       int line = ses->winy;
 
       while (line <= (int)(scr.rows - brl.textRows)) {
-        readScreen(0, line, scr.cols, 1, characters);
+        readScreenRow(line, scr.cols, characters);
 
         if (isAllSpaceCharacters(characters, scr.cols) == findBlankLine) {
           if (!findBlankLine) {
@@ -432,7 +432,7 @@ handleNavigationCommands (int command, void *data) {
       {
         ScreenCharacter characters[scr.cols];
         size_t length = 0;
-        readScreen(0, ses->winy, scr.cols, 1, characters);
+        readScreenRow(ses->winy, scr.cols, characters);
         while (length < scr.cols) {
           if (characters[length].text == WC_C(' ')) break;
           ++length;
@@ -466,10 +466,10 @@ handleNavigationCommands (int command, void *data) {
     }
 
     case BRL_CMD_CHRLT:
-      if (!moveWindowLeft(1)) alert(ALERT_BOUNCE);
+      if (!moveBrailleWindowLeft(1)) alert(ALERT_BOUNCE);
       break;
     case BRL_CMD_CHRRT:
-      if (!moveWindowRight(1)) alert(ALERT_BOUNCE);
+      if (!moveBrailleWindowRight(1)) alert(ALERT_BOUNCE);
       break;
 
     case BRL_CMD_HWINLT:
@@ -510,10 +510,9 @@ handleNavigationCommands (int command, void *data) {
 
         if (shiftBrailleWindowLeft(fullWindowShift)) {
           if (skipBlankBrailleWindows) {
-            int charCount;
-
             if (prefs.skipBlankBrailleWindowsMode == sbwEndOfLine) goto skipEndOfLine;
-            charCount = MIN(scr.cols, ses->winx+textCount);
+            int charCount = MIN(scr.cols, ses->winx+textCount);
+
             if (!showScreenCursor() ||
                 (scr.posy != ses->winy) ||
                 (scr.posx < 0) ||
@@ -521,7 +520,7 @@ handleNavigationCommands (int command, void *data) {
               int charIndex;
               ScreenCharacter characters[charCount];
 
-              readScreen(0, ses->winy, charCount, 1, characters);
+              readScreenRow(ses->winy, charCount, characters);
 
               for (charIndex=0; charIndex<charCount; charIndex+=1) {
                 wchar_t text = characters[charIndex].text;
@@ -539,7 +538,6 @@ handleNavigationCommands (int command, void *data) {
       wrapUp:
         if (ses->winy == 0) {
           ses->winx = oldX;
-
           alert(ALERT_BOUNCE);
           break;
         }
@@ -550,23 +548,18 @@ handleNavigationCommands (int command, void *data) {
 
       skipEndOfLine:
         if (skipBlankBrailleWindows && (prefs.skipBlankBrailleWindowsMode == sbwEndOfLine)) {
-          int charIndex;
           ScreenCharacter characters[scr.cols];
+          readScreenRow(ses->winy, scr.cols, characters);
+          int last = scr.cols;
 
-          readScreen(0, ses->winy, scr.cols, 1, characters);
-
-          for (charIndex=scr.cols-1; charIndex>0; charIndex-=1) {
-            wchar_t text = characters[charIndex].text;
-
-            if (text != WC_C(' ')) break;
+          while (last > 0) {
+            if (!isWordBreak(characters, ses->winy, --last)) break;
           }
 
-          if (showScreenCursor() && (scr.posy == ses->winy) && SCR_COLUMN_OK(scr.posx)) {
-            charIndex = MAX(charIndex, scr.posx);
-          }
-
-          if (charIndex < ses->winx) placeRightEdge(charIndex);
+          if (ses->winx > last) placeRightEdge(last);
         }
+
+        if (prefs.wordWrap) setWordWrapStart(ses->winx);
       }
 
       break;
@@ -685,7 +678,7 @@ handleNavigationCommands (int command, void *data) {
             arg = rescaleInteger(arg, BRL_MSK_ARG, scr.rows-1);
           }
           if (arg < scr.rows) {
-            slideWindowVertically(arg);
+            slideBrailleWindowVertically(arg);
             oldwiny = -1;
           } else {
             alert(ALERT_COMMAND_REJECTED);
