@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -1508,19 +1508,19 @@ activateBrailleDriver (int verify) {
     {
       const char *dev = brailleDevice;
 
-      if (isSerialDevice(&dev)) {
+      if (isSerialDeviceIdentifier(&dev)) {
         static const char *const serialDrivers[] = {
           "md", "pm", "ts", "ht", "bn", "al", "bm", "pg", "sk",
           NULL
         };
         autodetectableDrivers = serialDrivers;
-      } else if (isUsbDevice(&dev)) {
+      } else if (isUsbDeviceIdentifier(&dev)) {
         static const char *const usbDrivers[] = {
-          "al", "bm", "bn", "eu", "fs", "hd", "hm", "ht", "hw", "mt", "pg", "pm", "sk", "vo",
+          "al", "bm", "bn", "eu", "fs", "hd", "hm", "ht", "hw", "ic", "mt", "pg", "pm", "sk", "vo",
           NULL
         };
         autodetectableDrivers = usbDrivers;
-      } else if (isBluetoothDevice(&dev)) {
+      } else if (isBluetoothDeviceIdentifier(&dev)) {
         if (!(autodetectableDrivers = bthGetDriverCodes(dev, BLUETOOTH_DEVICE_NAME_OBTAIN_TIMEOUT))) {
           static const char *bluetoothDrivers[] = {
             "np", "ht", "al", "bm",
@@ -1584,6 +1584,7 @@ startBrailleDriver (void) {
   if (activateBrailleDriver(0)) {
     if (oldPreferencesEnabled) {
       if (!loadPreferencesFile(oldPreferencesFile)) resetPreferences();
+      setPreferenceOverrides();
       applyAllPreferences();
     } else {
       applyBraillePreferences();
@@ -1798,8 +1799,8 @@ ASYNC_ALARM_CALLBACK(handleAutospeakDelayAlarm) {
 
 static void
 beginAutospeakDelay (int duration) {
-  if (asyncSetAlarmIn(&autospeakDelayAlarm, duration,
-                      handleAutospeakDelayAlarm, NULL)) {
+  if (asyncNewRelativeAlarm(&autospeakDelayAlarm, duration,
+                            handleAutospeakDelayAlarm, NULL)) {
     spk.canAutospeak = 0;
   }
 }
@@ -2326,7 +2327,7 @@ tryPidFile (void) {
   } else if (errno == EEXIST) {
     return 0;
   } else {
-    asyncSetAlarmIn(NULL, PID_FILE_CREATE_RETRY_INTERVAL, retryPidFile, NULL);
+    asyncNewRelativeAlarm(NULL, PID_FILE_CREATE_RETRY_INTERVAL, retryPidFile, NULL);
   }
 
   return 1;
