@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -178,7 +178,7 @@ postprocessCommand (void *state, int command, int handled) {
               describeScreen(&description);
 
               if (description.number == scr.number) {
-                slideWindowVertically(description.posy);
+                slideBrailleWindowVertically(description.posy);
                 placeBrailleWindowHorizontally(description.posx);
               }
             }
@@ -555,12 +555,79 @@ getTimeFormattingData (TimeFormattingData *fmt) {
   fmt->meridian = getMeridianString(&fmt->components.hour);
 }
 
-void 
-slideWindowVertically (int y) {
-  if (y < ses->winy)
-    ses->winy = y;
-  else if (y >= (int)(ses->winy + brl.textRows))
-    ses->winy = y - (brl.textRows - 1);
+int
+isWordBreak (const ScreenCharacter *characters, int y, int x) {
+  if (!iswspace(characters[x].text)) return 0;
+  return !(showScreenCursor() && (y == scr.posy) && (x == scr.posx));
+}
+
+int
+getWordWrapLength (int row, int from, int count) {
+  int width = scr.cols;
+  if (from >= width) return 0;
+
+  int to = from + count;
+  if (to >= width) return width - from;
+
+  ScreenCharacter characters[width];
+  readScreenRow(row, width, characters);
+  int onWordBreak = iswspace(characters[to].text);
+
+  if (!onWordBreak) {
+    int index = to;
+
+    while (index > from) {
+      if (isWordBreak(characters, ses->winy, --index)) {
+        to = index;
+        onWordBreak = 1;
+        break;
+      }
+    }
+  }
+
+  if (onWordBreak) {
+    while (to < width) {
+      if (!isWordBreak(characters, ses->winy, to)) break;
+      to += 1;
+    }
+  }
+
+  return to - from;
+}
+
+void
+setWordWrapStart (int start) {
+  if (start < 0) start = 0;
+  ses->winx = start;
+
+  if (start > 0) {
+    int end = start + textCount;
+    if (end > scr.cols) end = scr.cols;
+
+    ScreenCharacter characters[end];
+    readScreenRow(ses->winy, end, characters);
+
+    while (end > 0) {
+      if (!isWordBreak(characters, ses->winy, --end)) {
+        end += 1;
+        break;
+      }
+    }
+
+    if (!isWordBreak(characters, ses->winy, start-1)) {
+      while (start < end) {
+        if (isWordBreak(characters, ses->winy, start)) break;
+        start += 1;
+      }
+    }
+
+    while (start < end) {
+      if (!isWordBreak(characters, ses->winy, start)) break;
+      start += 1;
+    }
+
+    if (start < end) ses->winx = start;
+  }
 }
 
 void 
@@ -599,7 +666,7 @@ placeBrailleWindowRight (void) {
 }
 
 int
-moveWindowLeft (unsigned int amount) {
+moveBrailleWindowLeft (unsigned int amount) {
   if (ses->winx < 1) return 0;
   if (amount < 1) return 0;
 
@@ -608,15 +675,13 @@ moveWindowLeft (unsigned int amount) {
 }
 
 int
-moveWindowRight (unsigned int amount) {
+moveBrailleWindowRight (unsigned int amount) {
+  if (amount < 1) return 0;
   int newx = ses->winx + amount;
+  if (newx >= scr.cols) return 0;
 
-  if ((newx > ses->winx) && (newx < scr.cols)) {
-    ses->winx = newx;
-    return 1;
-  }
-
-  return 0;
+  ses->winx = newx;
+  return 1;
 }
 
 int
@@ -637,6 +702,17 @@ shiftBrailleWindowLeft (unsigned int amount) {
       }
     }
 
+    if (first > 0) {
+      ScreenCharacter characters[reference];
+      readScreenRow(ses->winy, reference, characters);
+
+      if (!isWordBreak(characters, ses->winy, first-1)) {
+        while (!isWordBreak(characters, ses->winy, first)) first += 1;
+      }
+
+      while (isWordBreak(characters, ses->winy, first)) first += 1;
+    }
+
     if (first == reference) {
       if (!first) return 0;
       first -= 1;
@@ -647,16 +723,36 @@ shiftBrailleWindowLeft (unsigned int amount) {
   }
 #endif /* ENABLE_CONTRACTED_BRAILLE */
 
-  return moveWindowLeft(amount);
+  if (prefs.wordWrap) {
+    if (ses->winx < 1) return 0;
+    setWordWrapStart(ses->winx - amount);
+    return 1;
+  }
+
+  return moveBrailleWindowLeft(amount);
 }
 
 int
 shiftBrailleWindowRight (unsigned int amount) {
 #ifdef ENABLE_CONTRACTED_BRAILLE
-  if (isContracting()) amount = getContractedLength(amount);
+  if (isContracting()) {
+    amount = getContractedLength(amount);
+  } else
 #endif /* ENABLE_CONTRACTED_BRAILLE */
+  if (prefs.wordWrap) {
+    amount = getWordWrapLength(ses->winy, ses->winx, amount);
+  }
 
-  return moveWindowRight(amount);
+  return moveBrailleWindowRight(amount);
+}
+
+void
+slideBrailleWindowVertically (int y) {
+  if (y < ses->winy) {
+    ses->winy = y;
+  } else if (y >= (int)(ses->winy + brl.textRows)) {
+    ses->winy = y - (brl.textRows - 1);
+  }
 }
 
 static int
@@ -720,8 +816,8 @@ trackScreenCursor (int place) {
     ses->dcty = ses->trky;
 
     int delay = 250 << (prefs.cursorTrackingDelay - 1);
-    asyncSetAlarmIn(&delayedCursorTrackingAlarm, delay,
-                    handleDelayedCursorTrackingAlarm, NULL);
+    asyncNewRelativeAlarm(&delayedCursorTrackingAlarm, delay,
+                          handleDelayedCursorTrackingAlarm, NULL);
 
     return 1;
   }
@@ -737,7 +833,7 @@ trackScreenCursor (int place) {
       int length = scr.posx + 1;
       ScreenCharacter characters[length];
       int onspace = 1;
-      readScreen(0, ses->winy, length, 1, characters);
+      readScreenRow(ses->winy, length, characters);
       while (length) {
         if ((iswspace(characters[--length].text) != 0) != onspace) {
           if (onspace) {
@@ -775,7 +871,7 @@ trackScreenCursor (int place) {
     ses->winx += (scr.posx - ses->winx) / textCount * textCount;
   }
 
-  slideWindowVertically(scr.posy);
+  slideBrailleWindowVertically(scr.posy);
   return 1;
 }
 
@@ -821,7 +917,7 @@ volatile SpeechSynthesizer spk;
 void
 trackSpeech (void) {
   placeBrailleWindowHorizontally(spk.track.speechLocation % scr.cols);
-  slideWindowVertically(spk.track.firstLine + (spk.track.speechLocation / scr.cols));
+  slideBrailleWindowVertically(spk.track.firstLine + (spk.track.speechLocation / scr.cols));
   scheduleUpdate("speech tracked");
 }
 
@@ -985,6 +1081,14 @@ isSameAttributes (
   return character1->attributes == character2->attributes;
 }
 
+int
+isSameCharacter (
+  const ScreenCharacter *character1,
+  const ScreenCharacter *character2
+) {
+  return isSameText(character1, character2) && isSameAttributes(character1, character2);
+}
+
 int
 isSameRow (
   const ScreenCharacter *characters1,
