@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -23,6 +23,7 @@
 #include <errno.h>
  
 #include "log.h"
+#include "parse.h"
 #include "file.h"
 #include "ctb.h"
 #include "ctb_internal.h"
@@ -529,38 +530,6 @@ static DATA_OPERANDS_PROCESSOR(processContractionTableOperands) {
   return processDirectiveOperand(file, &directives, "contraction table directive", data);
 }
 
-int
-startContractionCommand (ContractionTable *table) {
-  if (!table->data.external.commandStarted) {
-    const char *command[] = {table->command, NULL};
-    HostCommandOptions options;
-
-    initializeHostCommandOptions(&options);
-    options.asynchronous = 1;
-    options.standardInput = &table->data.external.standardInput;
-    options.standardOutput = &table->data.external.standardOutput;
-
-    logMessage(LOG_DEBUG, "starting external contraction table: %s", table->command);
-    if (runHostCommand(command, &options) != 0) return 0;
-    logMessage(LOG_DEBUG, "external contraction table started: %s", table->command);
-
-    table->data.external.commandStarted = 1;
-  }
-
-  return 1;
-}
-
-void
-stopContractionCommand (ContractionTable *table) {
-  if (table->data.external.commandStarted) {
-    fclose(table->data.external.standardInput);
-    fclose(table->data.external.standardOutput);
-
-    logMessage(LOG_DEBUG, "external contraction table stopped: %s", table->command);
-    table->data.external.commandStarted = 0;
-  }
-}
-
 static void
 initializeCommonFields (ContractionTable *table) {
   table->characters.array = NULL;
@@ -580,38 +549,47 @@ initializeCommonFields (ContractionTable *table) {
   table->cache.offsets.count = 0;
 }
 
-ContractionTable *
-compileContractionTable (const char *fileName) {
-  ContractionTable *table = NULL;
-
-  if (testProgramPath(fileName)) {
-    if ((table = malloc(sizeof(*table)))) {
-      memset(table, 0, sizeof(*table));
-
-      if ((table->command = strdup(fileName))) {
-        initializeCommonFields(table);
-        table->data.external.commandStarted = 0;
-
-        table->data.external.input.buffer = NULL;
-        table->data.external.input.size = 0;
-
-        if (startContractionCommand(table)) {
-          return table;
-        }
-
-        free(table->command);
-      } else {
-        logMallocError();
-      }
-
-      free(table);
-    } else {
-      logMallocError();
-    }
-
-    return NULL;
+static void
+destroyCommonFields (ContractionTable *table) {
+  if (table->characters.array) {
+    free(table->characters.array);
+    table->characters.array = NULL;
   }
 
+  if (table->cache.input.characters) {
+    free(table->cache.input.characters);
+    table->cache.input.characters = NULL;
+  }
+
+  if (table->cache.output.cells) {
+    free(table->cache.output.cells);
+    table->cache.output.cells = NULL;
+  }
+
+  if (table->cache.offsets.array) {
+    free(table->cache.offsets.array);
+    table->cache.offsets.array = NULL;
+  }
+}
+
+static void
+destroyContractionTable_native (ContractionTable *table) {
+  destroyCommonFields(table);
+
+  if (table->data.internal.size) {
+    free(table->data.internal.header.fields);
+    free(table);
+  }
+}
+
+static const ContractionTableManagementMethods nativeManagementMethods = {
+  .destroy = destroyContractionTable_native
+};
+
+static ContractionTable *
+compileContractionTable_native (const char *fileName) {
+  ContractionTable *table = NULL;
+
   if (setTableDataVariables(CONTRACTION_TABLE_EXTENSION, CONTRACTION_SUBTABLE_EXTENSION)) {
     ContractionTableData ctd;
     memset(&ctd, 0, sizeof(ctd));
@@ -641,8 +619,9 @@ compileContractionTable (const char *fileName) {
           if (processDataFile(fileName, &parameters)) {
             if (saveCharacterTable(&ctd)) {
               if ((table = malloc(sizeof(*table)))) {
+                table->managementMethods = &nativeManagementMethods;
+                table->translationMethods = getContractionTableTranslationMethods_native();
                 initializeCommonFields(table);
-                table->command = NULL;
 
                 table->data.internal.header.fields = getContractionTableHeader(&ctd);
                 table->data.internal.size = getDataSize(ctd.area);
@@ -666,39 +645,182 @@ compileContractionTable (const char *fileName) {
   return table;
 }
 
+int
+startContractionCommand (ContractionTable *table) {
+  if (!table->data.external.commandStarted) {
+    const char *command[] = {table->data.external.command, NULL};
+    HostCommandOptions options;
+
+    initializeHostCommandOptions(&options);
+    options.asynchronous = 1;
+    options.standardInput = &table->data.external.standardInput;
+    options.standardOutput = &table->data.external.standardOutput;
+
+    logMessage(LOG_DEBUG, "starting external contraction table: %s", table->data.external.command);
+    if (runHostCommand(command, &options) != 0) return 0;
+    logMessage(LOG_DEBUG, "external contraction table started: %s", table->data.external.command);
+
+    table->data.external.commandStarted = 1;
+  }
+
+  return 1;
+}
+
 void
-destroyContractionTable (ContractionTable *table) {
-  if (table->characters.array) {
-    free(table->characters.array);
-    table->characters.array = NULL;
-  }
+stopContractionCommand (ContractionTable *table) {
+  if (table->data.external.commandStarted) {
+    fclose(table->data.external.standardInput);
+    fclose(table->data.external.standardOutput);
 
-  if (table->cache.input.characters) {
-    free(table->cache.input.characters);
-    table->cache.input.characters = NULL;
+    logMessage(LOG_DEBUG, "external contraction table stopped: %s", table->data.external.command);
+    table->data.external.commandStarted = 0;
   }
+}
 
-  if (table->cache.output.cells) {
-    free(table->cache.output.cells);
-    table->cache.output.cells = NULL;
-  }
+static void
+destroyContractionTable_external (ContractionTable *table) {
+  stopContractionCommand(table);
+  if (table->data.external.input.buffer) free(table->data.external.input.buffer);
+  free(table->data.external.command);
 
-  if (table->cache.offsets.array) {
-    free(table->cache.offsets.array);
-    table->cache.offsets.array = NULL;
-  }
+  destroyCommonFields(table);
+  free(table);
+}
+
+static const ContractionTableManagementMethods externalManagementMethods = {
+  .destroy = destroyContractionTable_external
+};
+
+static ContractionTable *
+compileContractionTable_external (const char *fileName) {
+  ContractionTable *table;
+
+  if ((table = malloc(sizeof(*table)))) {
+    memset(table, 0, sizeof(*table));
+
+    if ((table->data.external.command = strdup(fileName))) {
+      table->managementMethods = &externalManagementMethods;
+      table->translationMethods = getContractionTableTranslationMethods_external();
+      initializeCommonFields(table);
+
+      table->data.external.commandStarted = 0;
+
+      table->data.external.input.buffer = NULL;
+      table->data.external.input.size = 0;
+
+      if (startContractionCommand(table)) {
+        return table;
+      }
+
+      free(table->data.external.command);
+    } else {
+      logMallocError();
+    }
 
-  if (table->command) {
-    stopContractionCommand(table);
-    if (table->data.external.input.buffer) free(table->data.external.input.buffer);
-    free(table->command);
     free(table);
   } else {
-    if (table->data.internal.size) {
-      free(table->data.internal.header.fields);
-      free(table);
+    logMallocError();
+  }
+
+  return NULL;
+}
+
+#ifdef LOUIS_TABLES_DIRECTORY
+static void
+destroyContractionTable_louis (ContractionTable *table) {
+  free(table->data.louis.tableList);
+
+  destroyCommonFields(table);
+  free(table);
+}
+
+static const ContractionTableManagementMethods louisManagementMethods = {
+  .destroy = destroyContractionTable_louis
+};
+
+static ContractionTable *
+compileContractionTable_louis (const char *fileName) {
+  ContractionTable *table;
+
+  if ((table = malloc(sizeof(*table)))) {
+    memset(table, 0, sizeof(*table));
+
+    if ((table->data.louis.tableList = strdup(fileName))) {
+      table->managementMethods = &louisManagementMethods;
+      table->translationMethods = getContractionTableTranslationMethods_louis();
+      initializeCommonFields(table);
+
+      return table;
+    } else {
+      logMallocError();
+    }
+
+    free(table);
+  } else {
+    logMallocError();
+  }
+
+  return NULL;
+}
+#endif /* LOUIS_TABLES_DIRECTORY */
+
+typedef ContractionTable *ContractionTableCompileFunction (const char *fileName);
+
+typedef struct {
+  const char *qualifier;
+  ContractionTableCompileFunction *compile;
+  const char *directory;
+} ContractionTableQualifierEntry;
+
+static const ContractionTableQualifierEntry contractionTableQualifierTable[] = {
+#ifdef LOUIS_TABLES_DIRECTORY
+  { .qualifier = "louis",
+    .compile = &compileContractionTable_louis,
+    .directory = LOUIS_TABLES_DIRECTORY
+  },
+#endif /* LOUIS_TABLES_DIRECTORY */
+
+  { .qualifier = NULL }
+};
+
+const ContractionTableQualifierEntry *
+getContractionTableQualifierEntry (const char **fileName) {
+  const ContractionTableQualifierEntry *entry = contractionTableQualifierTable;
+
+  while (entry->qualifier) {
+    if (hasQualifier(fileName, entry->qualifier)) return entry;
+    entry += 1;
+  }
+
+  return NULL;
+}
+
+ContractionTable *
+compileContractionTable (const char *fileName) {
+  ContractionTableCompileFunction *compile = NULL;
+  const ContractionTableQualifierEntry *ctq = getContractionTableQualifierEntry(&fileName);
+
+  if (ctq) {
+    compile = ctq->compile;
+  } else {
+    if (!hasNoQualifier(fileName)) {
+      logMessage(LOG_ERR, "unsupported contraction table: %s", fileName);
+      return NULL;
+    }
+
+    if (testProgramPath(fileName)) {
+      compile = &compileContractionTable_external;
+    } else {
+      compile = &compileContractionTable_native;
     }
   }
+
+  return compile(fileName);
+}
+
+void
+destroyContractionTable (ContractionTable *table) {
+  table->managementMethods->destroy(table);
 }
 
 char *
@@ -708,13 +830,40 @@ ensureContractionTableExtension (const char *path) {
 
 char *
 makeContractionTablePath (const char *directory, const char *name) {
-  char *subdirectory = makePath(directory, CONTRACTION_TABLES_SUBDIRECTORY);
+  const char *qualifier = name;
+  const ContractionTableQualifierEntry *ctq = getContractionTableQualifierEntry(&name);
+  if (!ctq) hasQualifier(&name, NULL);
+  int qualifierLength = name - qualifier;
 
-  if (subdirectory) {
-    char *file = makeFilePath(subdirectory, name, CONTRACTION_TABLE_EXTENSION);
+  char *path;
+  const char *extension;
 
-    free(subdirectory);
-    if (file) return file;
+  if (ctq && ctq->directory) {
+    if (!(path = strdup(ctq->directory))) logMallocError();
+    extension = NULL;
+  } else {
+    path = makePath(directory, CONTRACTION_TABLES_SUBDIRECTORY);
+    extension = CONTRACTION_TABLE_EXTENSION;
+  }
+
+  if (path) {
+    char *file = makeFilePath(path, name, extension);
+
+    free(path);
+    path = NULL;
+
+    if (file) {
+      if (qualifierLength) {
+        char buffer[qualifierLength + strlen(file) + 1];
+        snprintf(buffer, sizeof(buffer), "%.*s%s",
+                 qualifierLength, qualifier, file);
+
+        free(file);
+        if (!(file = strdup(buffer))) logMallocError();
+      }
+
+      if (file) return file;
+    }
   }
 
   return NULL;
