@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -119,7 +119,20 @@ typedef struct {
   const ContractionTableRule *always;
 } CharacterEntry;
 
+typedef struct {
+  void (*destroy) (ContractionTable *table);
+} ContractionTableManagementMethods;
+
+typedef struct ContractionTableTranslationMethodsStruct ContractionTableTranslationMethods;
+typedef const ContractionTableTranslationMethods *GetContractionTableTranslationMethodsFunction (void);
+extern GetContractionTableTranslationMethodsFunction getContractionTableTranslationMethods_native;
+extern GetContractionTableTranslationMethodsFunction getContractionTableTranslationMethods_external;
+extern GetContractionTableTranslationMethodsFunction getContractionTableTranslationMethods_louis;
+
 struct ContractionTableStruct {
+  const ContractionTableManagementMethods *managementMethods;
+  const ContractionTableTranslationMethods *translationMethods;
+
   struct {
     CharacterEntry *array;
     int size;
@@ -152,8 +165,6 @@ struct ContractionTableStruct {
     unsigned char capitalizationMode;
   } cache;
 
-  char *command;
-
   union {
     struct {
       union {
@@ -165,15 +176,22 @@ struct ContractionTableStruct {
     } internal;
 
     struct {
-      unsigned commandStarted:1;
+      char *command;
       FILE *standardInput;
       FILE *standardOutput;
+      unsigned commandStarted:1;
 
       struct {
         char *buffer;
         size_t size;
       } input;
     } external;
+
+#ifdef LOUIS_TABLES_DIRECTORY
+    struct {
+      char *tableList;
+    } louis;
+#endif /* LOUIS_TABLES_DIRECTORY */
   } data;
 };
 
