@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -65,14 +65,16 @@ extern "C" {
 #define DEFAULT_CAPITALS_VISIBLE_TIME 60	/* for blinking caps */
 #define DEFAULT_CAPITALS_INVISIBLE_TIME 20
 
-#define DEFAULT_SKIP_IDENTICAL_LINES 0		/* 1 = skip all identical lines after first */
-#define DEFAULT_SKIP_BLANK_BRAILLE_WINDOWS 0       /* 1 = skip blank windows */
+#define DEFAULT_WORD_WRAP 0
+#define DEFAULT_SKIP_IDENTICAL_LINES 0
+#define DEFAULT_SKIP_BLANK_BRAILLE_WINDOWS 0
 #define DEFAULT_SKIP_BLANK_BRAILLE_WINDOWS_MODE sbwEndOfLine
-#define DEFAULT_SLIDING_BRAILLE_WINDOW 0		/* 1 for on, 0 for off */
+#define DEFAULT_SLIDING_BRAILLE_WINDOW 0
 #define DEFAULT_EAGER_SLIDING_BRAILLE_WINDOW 0
 #define DEFAULT_BRAILLE_WINDOW_OVERLAP 0
 #define DEFAULT_CURSOR_TRACKING_DELAY ctd250ms
 
+#define DEFAULT_TRACK_SCREEN_SCROLL 0		/* 1 for on, 0 for off */
 #define DEFAULT_TRACK_SCREEN_POINTER 0		/* 1 for on, 0 for off */
 #define DEFAULT_HIGHLIGHT_BRAILLE_WINDOW_LOCATION 0		/* 1 for on, 0 for off */
 
