@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -167,31 +167,6 @@ resolveDeviceName (const char *const *names, const char *description) {
   return device;
 }
 
-int
-isQualifiedDevice (const char **identifier, const char *qualifier) {
-  const char *delimiter = strchr(*identifier, PARAMETER_QUALIFIER_CHARACTER);
-
-  if (delimiter) {
-    size_t count = delimiter - *identifier;
-
-    if (!qualifier) return 1;
-
-    if (count) {
-      if (strncasecmp(*identifier, qualifier, count) == 0) {
-        *identifier += count + 1;
-        return 1;
-      }
-    }
-  }
-
-  return 0;
-}
-
-int
-isUnqualifiedDevice (const char *identifier) {
-  return !isQualifiedDevice(&identifier, NULL);
-}
-
 char **
 getDeviceParameters (const char *const *names, const char *identifier) {
   char parameters[strlen(names[0]) + 1 + strlen(identifier) + 1];
