@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -23,7 +23,7 @@
 #include "log.h"
 #include "io_generic.h"
 #include "gio_internal.h"
-#include "device.h"
+#include "parse.h"
 
 struct GioHandleStruct {
   int place_holder;
@@ -69,7 +69,7 @@ isNullSupported (const GioDescriptor *descriptor) {
 
 static int
 testNullIdentifier (const char **identifier) {
-  return isQualifiedDevice(identifier, "null");
+  return hasQualifier(identifier, "null");
 }
 
 static const GioOptions *
