@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -76,8 +76,8 @@ setAutoreleaseAlarm (KeyTable *table) {
   } else if (table->autorelease.alarm) {
     asyncResetAlarmIn(table->autorelease.alarm, table->autorelease.time);
   } else {
-    asyncSetAlarmIn(&table->autorelease.alarm, table->autorelease.time,
-                    handleKeyAutoreleaseAlarm, table);
+    asyncNewRelativeAlarm(&table->autorelease.alarm, table->autorelease.time,
+                          handleKeyAutoreleaseAlarm, table);
   }
 }
 
@@ -389,8 +389,8 @@ ASYNC_ALARM_CALLBACK(handleLongPressAlarm) {
 
 static void
 setLongPressAlarm (KeyTable *table, unsigned char when) {
-  asyncSetAlarmIn(&table->longPress.alarm, PREFERENCES_TIME(when),
-                  handleLongPressAlarm, table);
+  asyncNewRelativeAlarm(&table->longPress.alarm, PREFERENCES_TIME(when),
+                        handleLongPressAlarm, table);
 }
 
 static int
