@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -341,6 +341,7 @@ changedStatusField (unsigned char index, unsigned char setting) {
       if (!haveStatusCells()) return 0;
       if (!braille->statusFields) return 0;
       if (*braille->statusFields != sfGeneric) return 0;
+      /* fall through */
 
     case sfEnd:
       if (prefs.statusFields[index+1] != sfEnd) return 0;
@@ -716,6 +717,10 @@ makePreferencesMenu (void) {
   {
     SUBMENU(navigationSubmenu, rootMenu, strtext("Navigation Options"));
 
+    {
+      NAME(strtext("Word Wrap"));
+      ITEM(newBooleanMenuItem(navigationSubmenu, &prefs.wordWrap, &itemName));
+    }
     {
       NAME(strtext("Skip Identical Lines"));
       ITEM(newBooleanMenuItem(navigationSubmenu, &prefs.skipIdenticalLines, &itemName));
@@ -767,6 +772,11 @@ makePreferencesMenu (void) {
       ITEM(newEnumeratedMenuItem(navigationSubmenu, &prefs.cursorTrackingDelay, &itemName, strings));
     }
 
+    {
+      NAME(strtext("Track Screen Scroll"));
+      ITEM(newBooleanMenuItem(navigationSubmenu, &prefs.trackScreenScroll, &itemName));
+    }
+
 #ifdef HAVE_LIBGPM
     {
       NAME(strtext("Track Screen Pointer"));
