@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -249,6 +249,12 @@ const PreferenceEntry preferenceTable[] = {
     .setting = &prefs.capitalsInvisibleTime
   },
 
+  { .name = "word-wrap",
+    .defaultValue = DEFAULT_WORD_WRAP,
+    .settingNames = &preferenceStringTable_boolean,
+    .setting = &prefs.wordWrap
+  },
+
   { .name = "skip-identical-lines",
     .defaultValue = DEFAULT_SKIP_IDENTICAL_LINES,
     .settingNames = &preferenceStringTable_boolean,
@@ -290,6 +296,12 @@ const PreferenceEntry preferenceTable[] = {
     .setting = &prefs.cursorTrackingDelay
   },
 
+  { .name = "track-screen-scroll",
+    .defaultValue = DEFAULT_TRACK_SCREEN_SCROLL,
+    .settingNames = &preferenceStringTable_boolean,
+    .setting = &prefs.trackScreenScroll
+  },
+
   { .name = "track-screen-pointer",
     .defaultValue = DEFAULT_TRACK_SCREEN_POINTER,
     .settingNames = &preferenceStringTable_boolean,
