@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -40,7 +40,7 @@
 #include "routing.h"
 
 /*
- * These control the performance of cursor routing.  The optimal settings
+ * These control the performance of cursor crd.  The optimal settings
  * will depend heavily on system load, etc.  See the documentation for
  * further details.
  * NOTE: if you try to route the cursor to an invalid place, BRLTTY won't
@@ -58,22 +58,38 @@ typedef enum {
 
 typedef struct {
 #ifdef HAVE_SIGNAL_H
-  sigset_t signalMask;
+  struct {
+    sigset_t mask;
+  } signal;
 #endif /* HAVE_SIGNAL_H */
 
-  int screenNumber;
-  int screenRows;
-  int screenColumns;
+  struct {
+    int number;
+    int width;
+    int height;
+  } screen;
 
-  int verticalDelta;
-  ScreenCharacter *rowBuffer;
+  struct {
+    int scroll;
+    int row;
+    ScreenCharacter *buffer;
+  } vertical;
 
-  int cury, curx;
-  int oldy, oldx;
+  struct {
+    int column;
+    int row;
+  } current;
 
-  long timeSum;
-  int timeCount;
-} RoutingData;
+  struct {
+    int column;
+    int row;
+  } previous;
+
+  struct {
+    long sum;
+    int count;
+  } time;
+} CursorRoutingData;
 
 typedef enum {
   CURSOR_DIR_LEFT,
@@ -99,181 +115,149 @@ typedef enum {
   CURSOR_AXIS_VERTICAL
 } CursorAxis;
 
-static void
-adjustHorizontalCoordinate (int *y, int *x, int amount) {
-  *x += amount;
-}
-
-static void
-adjustVerticalCoordinate (int *y, int *x, int amount) {
-  *y += amount;
-}
-
 typedef struct {
   const CursorDirectionEntry *forward;
   const CursorDirectionEntry *backward;
-  void (*adjustCoordinate) (int *y, int *x, int amount);
 } CursorAxisEntry;
 
 static const CursorAxisEntry cursorAxisTable[] = {
   [CURSOR_AXIS_HORIZONTAL] = {
     .forward  = &cursorDirectionTable[CURSOR_DIR_RIGHT],
-    .backward = &cursorDirectionTable[CURSOR_DIR_LEFT],
-    .adjustCoordinate = adjustHorizontalCoordinate
+    .backward = &cursorDirectionTable[CURSOR_DIR_LEFT]
   }
   ,
   [CURSOR_AXIS_VERTICAL] = {
     .forward  = &cursorDirectionTable[CURSOR_DIR_DOWN],
-    .backward = &cursorDirectionTable[CURSOR_DIR_UP],
-    .adjustCoordinate = adjustVerticalCoordinate
+    .backward = &cursorDirectionTable[CURSOR_DIR_UP]
   }
 };
 
 #define logRouting(...) logMessage(LOG_CATEGORY(CURSOR_ROUTING), __VA_ARGS__)
 
 static int
-readScreenRow (RoutingData *routing, ScreenCharacter *buffer, int row) {
-  if (!buffer) buffer = routing->rowBuffer;
-  return readScreen(0, row, routing->screenColumns, 1, buffer);
+readRow (CursorRoutingData *crd, ScreenCharacter *buffer, int row) {
+  if (!buffer) buffer = crd->vertical.buffer;
+  if (readScreenRow(row, crd->screen.width, buffer)) return 1;
+  logRouting("read failed: row=%d", row);
+  return 0;
 }
 
 static int
-getCurrentPosition (RoutingData *routing) {
+getCurrentPosition (CursorRoutingData *crd) {
   ScreenDescription description;
   describeScreen(&description);
 
-  if (description.number != routing->screenNumber) {
-    logRouting("screen changed: num=%d", description.number);
-    routing->screenNumber = description.number;
+  if (description.number != crd->screen.number) {
+    logRouting("screen changed: %d -> %d", crd->screen.number, description.number);
+    crd->screen.number = description.number;
     return 0;
   }
 
-  if (!routing->rowBuffer) {
-    routing->screenRows = description.rows;
-    routing->screenColumns = description.cols;
-    routing->verticalDelta = 0;
+  if (!crd->vertical.buffer) {
+    crd->screen.width = description.cols;
+    crd->screen.height = description.rows;
+    crd->vertical.scroll = 0;
 
-    if (!(routing->rowBuffer = calloc(routing->screenColumns, sizeof(*routing->rowBuffer)))) {
+    if (!(crd->vertical.buffer = malloc(ARRAY_SIZE(crd->vertical.buffer, crd->screen.width)))) {
       logMallocError();
       goto error;
     }
 
     logRouting("screen: num=%d cols=%d rows=%d",
-               routing->screenNumber,
-               routing->screenColumns, routing->screenRows);
-  } else if ((routing->screenRows != description.rows) ||
-             (routing->screenColumns != description.cols)) {
-    logRouting("size changed: cols=%d rows=%d",
+               crd->screen.number,
+               crd->screen.width, crd->screen.height);
+  } else if ((crd->screen.width != description.cols) ||
+             (crd->screen.height != description.rows)) {
+    logRouting("size changed: %dx%d -> %dx%d",
+               crd->screen.width, crd->screen.height,
                description.cols, description.rows);
     goto error;
   }
 
-  routing->cury = description.posy - routing->verticalDelta;
-  routing->curx = description.posx;
-
-  if (readScreenRow(routing, NULL, description.posy)) return 1;
-  logRouting("read failed: row=%d", description.posy);
+  crd->current.row = description.posy + crd->vertical.scroll;
+  crd->current.column = description.posx;
+  return 1;
 
 error:
-  routing->screenNumber = -1;
+  crd->screen.number = -1;
   return 0;
 }
 
 static void
-moveCursor (RoutingData *routing, const CursorDirectionEntry *direction) {
-#ifdef SIGUSR1
-  sigset_t oldMask;
-  sigprocmask(SIG_BLOCK, &routing->signalMask, &oldMask);
-#endif /* SIGUSR1 */
+handleVerticalScrolling (CursorRoutingData *crd, int direction) {
+  int firstRow = crd->vertical.row;
+  int currentRow = firstRow;
 
-  logRouting("move: %s", direction->name);
-  insertScreenKey(direction->key);
+  int bestRow = firstRow;
+  int bestLength = 0;
 
-#ifdef SIGUSR1
-  sigprocmask(SIG_SETMASK, &oldMask, NULL);
-#endif /* SIGUSR1 */
+  do {
+    ScreenCharacter buffer[crd->screen.width];
+    if (!readRow(crd, buffer, currentRow)) break;
+
+    int length;
+    {
+      int before = crd->current.column;
+      int after = before;
+
+      while (buffer[before].text == crd->vertical.buffer[before].text)
+        if (--before < 0)
+          break;
+
+      while (buffer[after].text == crd->vertical.buffer[after].text)
+        if (++after >= crd->screen.width)
+          break;
+
+      length = after - before - 1;
+    }
+
+    if (length > bestLength) {
+      bestRow = currentRow;
+      if ((bestLength = length) == crd->screen.width) break;
+    }
+
+    currentRow -= direction;
+  } while ((currentRow >= 0) && (currentRow < crd->screen.height));
+
+  int delta = bestRow - firstRow;
+  crd->vertical.scroll -= delta;
+  crd->current.row -= delta;
 }
 
 static int
-awaitCursorMotion (RoutingData *routing, int direction, const CursorAxisEntry *axis) {
-  int moved = 0;
-  long int timeout = routing->timeSum / routing->timeCount;
+awaitCursorMotion (CursorRoutingData *crd, int direction) {
+  crd->previous.column = crd->current.column;
+  crd->previous.row = crd->current.row;
+
   TimeValue start;
-
-  int trgy = routing->cury;
-  int trgx = routing->curx;
-
-  routing->oldy = routing->cury;
-  routing->oldx = routing->curx;
-
-  axis->adjustCoordinate(&trgy, &trgx, direction);
   getMonotonicTime(&start);
 
+  int moved = 0;
+  long int timeout = crd->time.sum / crd->time.count;
+
   while (1) {
-    long int time;
-    TimeValue now;
-
-    int oldy;
-    int oldx;
-
     asyncWait(ROUTING_INTERVAL);
+
+    TimeValue now;
     getMonotonicTime(&now);
-    time = millisecondsBetween(&start, &now) + 1;
+    long int time = millisecondsBetween(&start, &now) + 1;
 
-    {
-      int row = routing->cury + routing->verticalDelta;
-      int bestRow = row;
-      int bestLength = 0;
+    int oldy = crd->current.row;
+    int oldx = crd->current.column;
+    if (!getCurrentPosition(crd)) return 0;
 
-      do {
-        ScreenCharacter buffer[routing->screenColumns];
-        if (!readScreenRow(routing, buffer, row)) break;
-
-        {
-          int before = routing->curx;
-          int after = before;
-
-          while (buffer[before].text == routing->rowBuffer[before].text)
-            if (--before < 0)
-              break;
-
-          while (buffer[after].text == routing->rowBuffer[after].text)
-            if (++after >= routing->screenColumns)
-              break;
-
-          {
-            int length = after - before - 1;
-            if (length > bestLength) {
-              bestRow = row;
-              if ((bestLength = length) == routing->screenColumns) break;
-            }
-          }
-        }
-
-        row -= direction;
-      } while ((row >= 0) && (row < routing->screenRows));
-
-      routing->verticalDelta = bestRow - routing->cury;
-    }
-
-    oldy = routing->cury;
-    oldx = routing->curx;
-    if (!getCurrentPosition(routing)) return 0;
-
-    if ((routing->cury != oldy) || (routing->curx != oldx)) {
+    if ((crd->current.row != oldy) || (crd->current.column != oldx)) {
       logRouting("moved: [%d,%d] -> [%d,%d] (%ldms)",
-                 oldx, oldy, routing->curx, routing->cury, time);
+                 oldx, oldy, crd->current.column, crd->current.row, time);
 
       if (!moved) {
         moved = 1;
         timeout = (time * 2) + 1;
 
-        routing->timeSum += time * 8;
-        routing->timeCount += 1;
+        crd->time.sum += time * 8;
+        crd->time.count += 1;
       }
 
-      if ((routing->cury == trgy) && (routing->curx == trgx)) break;
-
       if (ROUTING_INTERVAL) {
         start = now;
       } else {
@@ -281,21 +265,41 @@ awaitCursorMotion (RoutingData *routing, int direction, const CursorAxisEntry *a
         getMonotonicTime(&start);
       }
     } else if (time > timeout) {
-      if (!moved) logRouting("timed out: %ldms", timeout);
       break;
     }
   }
 
+  handleVerticalScrolling(crd, direction);
+  return 1;
+}
+
+static int
+moveCursor (CursorRoutingData *crd, const CursorDirectionEntry *direction) {
+  crd->vertical.row = crd->current.row - crd->vertical.scroll;
+  if (!readRow(crd, NULL, crd->vertical.row)) return 0;
+
+#ifdef SIGUSR1
+  sigset_t oldMask;
+  sigprocmask(SIG_BLOCK, &crd->signal.mask, &oldMask);
+#endif /* SIGUSR1 */
+
+  logRouting("move: %s", direction->name);
+  insertScreenKey(direction->key);
+
+#ifdef SIGUSR1
+  sigprocmask(SIG_SETMASK, &oldMask, NULL);
+#endif /* SIGUSR1 */
+
   return 1;
 }
 
 static RoutingResult
-adjustCursorPosition (RoutingData *routing, int where, int trgy, int trgx, const CursorAxisEntry *axis) {
+adjustCursorPosition (CursorRoutingData *crd, int where, int trgy, int trgx, const CursorAxisEntry *axis) {
   logRouting("to: [%d,%d]", trgx, trgy);
 
   while (1) {
-    int dify = trgy - routing->cury;
-    int difx = (trgx < 0)? 0: (trgx - routing->curx);
+    int dify = trgy - crd->current.row;
+    int difx = (trgx < 0)? 0: (trgx - crd->current.column);
     int dir;
 
     /* determine which direction the cursor needs to move in */
@@ -308,32 +312,32 @@ adjustCursorPosition (RoutingData *routing, int where, int trgy, int trgx, const
     }
 
     /* tell the cursor to move in the needed direction */
-    moveCursor(routing, ((dir > 0)? axis->forward: axis->backward));
-    if (!awaitCursorMotion(routing, dir, axis)) return CRR_FAIL;
+    if (!moveCursor(crd, ((dir > 0)? axis->forward: axis->backward))) return CRR_FAIL;
+    if (!awaitCursorMotion(crd, dir)) return CRR_FAIL;
 
-    if (routing->cury != routing->oldy) {
-      if (routing->oldy != trgy) {
-        if (((routing->cury - routing->oldy) * dir) > 0) {
-          int dif = trgy - routing->cury;
+    if (crd->current.row != crd->previous.row) {
+      if (crd->previous.row != trgy) {
+        if (((crd->current.row - crd->previous.row) * dir) > 0) {
+          int dif = trgy - crd->current.row;
           if ((dif * dify) >= 0) continue;
           if (where > 0) {
-            if (routing->cury > trgy) return CRR_NEAR;
+            if (crd->current.row > trgy) return CRR_NEAR;
           } else if (where < 0) {
-            if (routing->cury < trgy) return CRR_NEAR;
+            if (crd->current.row < trgy) return CRR_NEAR;
           } else {
             if ((dif * dif) < (dify * dify)) return CRR_NEAR;
           }
         }
       }
-    } else if (routing->curx != routing->oldx) {
-      if (((routing->curx - routing->oldx) * dir) > 0) {
-        int dif = trgx - routing->curx;
-        if (routing->cury != trgy) continue;
+    } else if (crd->current.column != crd->previous.column) {
+      if (((crd->current.column - crd->previous.column) * dir) > 0) {
+        int dif = trgx - crd->current.column;
+        if (crd->current.row != trgy) continue;
         if ((dif * difx) >= 0) continue;
         if (where > 0) {
-          if (routing->curx > trgx) return CRR_NEAR;
+          if (crd->current.column > trgx) return CRR_NEAR;
         } else if (where < 0) {
-          if (routing->curx < trgx) return CRR_NEAR;
+          if (crd->current.column < trgx) return CRR_NEAR;
         } else {
           if ((dif * dif) < (difx * difx)) return CRR_NEAR;
         }
@@ -346,19 +350,19 @@ adjustCursorPosition (RoutingData *routing, int where, int trgy, int trgx, const
      * try going back to the previous position since it was obviously
      * the nearest ever reached.
      */
-    moveCursor(routing, ((dir > 0)? axis->backward: axis->forward));
-    return awaitCursorMotion(routing, -dir, axis)? CRR_NEAR: CRR_FAIL;
+    if (!moveCursor(crd, ((dir > 0)? axis->backward: axis->forward))) return CRR_FAIL;
+    return awaitCursorMotion(crd, -dir)? CRR_NEAR: CRR_FAIL;
   }
 }
 
 static RoutingResult
-adjustCursorHorizontally (RoutingData *routing, int where, int row, int column) {
-  return adjustCursorPosition(routing, where, row, column, &cursorAxisTable[CURSOR_AXIS_HORIZONTAL]);
+adjustCursorHorizontally (CursorRoutingData *crd, int where, int row, int column) {
+  return adjustCursorPosition(crd, where, row, column, &cursorAxisTable[CURSOR_AXIS_HORIZONTAL]);
 }
 
 static RoutingResult
-adjustCursorVertically (RoutingData *routing, int where, int row) {
-  return adjustCursorPosition(routing, where, row, -1, &cursorAxisTable[CURSOR_AXIS_VERTICAL]);
+adjustCursorVertically (CursorRoutingData *crd, int where, int row) {
+  return adjustCursorPosition(crd, where, row, -1, &cursorAxisTable[CURSOR_AXIS_VERTICAL]);
 }
 
 typedef struct {
@@ -369,32 +373,32 @@ typedef struct {
 
 static RoutingStatus
 routeCursor (const RoutingParameters *parameters) {
-  RoutingData routing;
+  CursorRoutingData crd;
 
 #ifdef SIGUSR1
   /* Set up the signal mask. */
-  sigemptyset(&routing.signalMask);
-  sigaddset(&routing.signalMask, SIGUSR1);
-  sigprocmask(SIG_UNBLOCK, &routing.signalMask, NULL);
+  sigemptyset(&crd.signal.mask);
+  sigaddset(&crd.signal.mask, SIGUSR1);
+  sigprocmask(SIG_UNBLOCK, &crd.signal.mask, NULL);
 #endif /* SIGUSR1 */
 
   /* initialize the routing data structure */
-  routing.screenNumber = parameters->screen;
-  routing.rowBuffer = NULL;
-  routing.timeSum = ROUTING_TIMEOUT;
-  routing.timeCount = 1;
+  crd.screen.number = parameters->screen;
+  crd.vertical.buffer = NULL;
+  crd.time.sum = ROUTING_TIMEOUT;
+  crd.time.count = 1;
 
-  if (getCurrentPosition(&routing)) {
-    logRouting("from: [%d,%d]", routing.curx, routing.cury);
+  if (getCurrentPosition(&crd)) {
+    logRouting("from: [%d,%d]", crd.current.column, crd.current.row);
 
     if (parameters->column < 0) {
-      adjustCursorVertically(&routing, 0, parameters->row);
+      adjustCursorVertically(&crd, 0, parameters->row);
     } else {
-      if (adjustCursorVertically(&routing, -1, parameters->row) != CRR_FAIL) {
-        if (adjustCursorHorizontally(&routing, 0, parameters->row, parameters->column) == CRR_NEAR) {
-          if (routing.cury < parameters->row) {
-            if (adjustCursorVertically(&routing, 1, routing.cury+1) != CRR_FAIL) {
-              adjustCursorHorizontally(&routing, 0, parameters->row, parameters->column);
+      if (adjustCursorVertically(&crd, -1, parameters->row) != CRR_FAIL) {
+        if (adjustCursorHorizontally(&crd, 0, parameters->row, parameters->column) == CRR_NEAR) {
+          if (crd.current.row < parameters->row) {
+            if (adjustCursorVertically(&crd, 1, crd.current.row+1) != CRR_FAIL) {
+              adjustCursorHorizontally(&crd, 0, parameters->row, parameters->column);
             }
           }
         }
@@ -402,11 +406,11 @@ routeCursor (const RoutingParameters *parameters) {
     }
   }
 
-  if (routing.rowBuffer) free(routing.rowBuffer);
+  if (crd.vertical.buffer) free(crd.vertical.buffer);
 
-  if (routing.screenNumber != parameters->screen) return ROUTING_ERROR;
-  if (routing.cury != parameters->row) return ROUTING_WRONG_ROW;
-  if ((parameters->column >= 0) && (routing.curx != parameters->column)) return ROUTING_WRONG_COLUMN;
+  if (crd.screen.number != parameters->screen) return ROUTING_ERROR;
+  if (crd.current.row != parameters->row) return ROUTING_WRONG_ROW;
+  if ((parameters->column >= 0) && (crd.current.column != parameters->column)) return ROUTING_WRONG_COLUMN;
   return ROUTING_DONE;
 }
 
