@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -47,6 +47,16 @@ extern int userVirtualTerminal (int number);
 extern int handleScreenCommands (int command, void *data);
 extern KeyTableCommandContext getScreenCommandContext (void);
 
+static inline int
+readScreenRows (int row, int width, int height, ScreenCharacter *buffer) {
+  return readScreen(0, row, width, height, buffer);
+}
+
+static inline int
+readScreenRow (int row, int width, ScreenCharacter *buffer) {
+  return readScreenRows(row, width, 1, buffer);
+}
+
 /* Routines which apply to the routing screen.
  * An extra `thread' for the cursor routing subprocess.
  * This is needed because the forked subprocess shares its parent's
