@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -866,12 +866,12 @@ serialGetStream (SerialDevice *serial) {
 }
 
 int
-isSerialDevice (const char **identifier) {
+isSerialDeviceIdentifier (const char **identifier) {
 #ifdef ALLOW_DOS_DEVICE_NAMES
   if (isDosDevice(*identifier, "COM")) return 1;
 #endif /* ALLOW_DOS_DEVICE_NAMES */
 
-  if (isQualifiedDevice(identifier, "serial")) return 1;
-  if (isUnqualifiedDevice(*identifier)) return 1;
+  if (hasQualifier(identifier, "serial")) return 1;
+  if (hasNoQualifier(*identifier)) return 1;
   return 0;
 }
