@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -23,6 +23,7 @@
 
 #include "parameters.h"
 #include "log.h"
+#include "alert.h"
 #include "report.h"
 #include "strfmt.h"
 #include "update.h"
@@ -305,6 +306,81 @@ showInfo (void) {
   return writeBrailleText(mode, text);
 }
 
+static int
+saveScreenCharacters (
+  ScreenCharacter **buffer, size_t *size,
+  const ScreenCharacter *characters, size_t count
+) {
+  size_t newSize = count * sizeof(*characters);
+
+  if (newSize > *size) {
+    ScreenCharacter *newBuffer = malloc(newSize);
+
+    if (!newBuffer) {
+      logMallocError();
+      return 0;
+    }
+
+    if (*buffer) free(*buffer);
+    *buffer = newBuffer;
+    *size = newSize;
+  }
+
+  memcpy(*buffer, characters, newSize);
+  return 1;
+}
+
+static void
+checkScreenScroll (int track) {
+  const int rowCount = 3;
+
+  static int oldScreen = -1;
+  static int oldRow = -1;
+  static int oldWidth = 0;
+  static size_t oldSize = 0;
+  static ScreenCharacter *oldCharacters = NULL;
+
+  int newScreen = scr.number;
+  int newWidth = scr.cols;
+  size_t newCount = newWidth * rowCount;
+  ScreenCharacter newCharacters[newCount];
+
+  int newRow = ses->winy;
+  int newTop = newRow - (rowCount - 1);
+
+  if (newTop < 0) {
+    newCount = 0;
+  } else {
+    readScreenRows(newTop, newWidth, rowCount, newCharacters);
+
+    if (track && prefs.trackScreenScroll && oldCharacters &&
+        (newScreen == oldScreen) && (newWidth == oldWidth) &&
+        (newRow == oldRow)) {
+      while (newTop > 0) {
+        if ((scr.posy >= newTop) && (scr.posy <= newRow)) break;
+
+        if (isSameRow(oldCharacters, newCharacters, newCount, isSameCharacter)) {
+          if (newRow != ses->winy) {
+            ses->winy = newRow;
+            alert(ALERT_SCROLL_UP);
+          }
+
+          break;
+        }
+
+        readScreenRows(--newTop, newWidth, rowCount, newCharacters);
+        newRow -= 1;
+      }
+    }
+  }
+
+  if (saveScreenCharacters(&oldCharacters, &oldSize, newCharacters, newCount)) {
+    oldScreen = newScreen;
+    oldRow = ses->winy;
+    oldWidth = newWidth;
+  }
+}
+
 #ifdef ENABLE_SPEECH_SUPPORT
 static int wasAutospeaking;
 
@@ -324,7 +400,7 @@ autospeak (AutospeakMode mode) {
   int newWidth = scr.cols;
   ScreenCharacter newCharacters[newWidth];
 
-  readScreen(0, ses->winy, newWidth, 1, newCharacters);
+  readScreenRow(ses->winy, newWidth, newCharacters);
 
   if (!spk.track.isActive) {
     const ScreenCharacter *characters = newCharacters;
@@ -517,29 +593,13 @@ autospeak (AutospeakMode mode) {
     }
   }
 
-  {
-    size_t newSize = newWidth * sizeof(*oldCharacters);
-
-    if (newSize > oldSize) {
-      ScreenCharacter *newBuffer = malloc(newSize);
-
-      if (!newBuffer) {
-        logMallocError();
-        return;
-      }
-
-      oldCharacters = newBuffer;
-      oldSize = newSize;
-    }
-
-    memcpy(oldCharacters, newCharacters, newSize);
+  if (saveScreenCharacters(&oldCharacters, &oldSize, newCharacters, newWidth)) {
+    oldScreen = newScreen;
+    oldX = newX;
+    oldY = newY;
+    oldWidth = newWidth;
+    cursorAssumedStable = 0;
   }
-
-  oldScreen = newScreen;
-  oldX = newX;
-  oldY = newY;
-  oldWidth = newWidth;
-  cursorAssumedStable = 0;
 }
 
 void
@@ -581,8 +641,6 @@ writeBrailleWindow (BrailleDisplay *brl, const wchar_t *text) {
 
 static void
 doUpdate (void) {
-  int screenPointerMoved = 0;
-
   logMessage(LOG_CATEGORY(UPDATE_EVENTS), "starting");
   unrequireAllBlinkDescriptors();
   refreshScreen();
@@ -621,6 +679,9 @@ doUpdate (void) {
     }
   }
 
+  int screenPointerHasMoved = 0;
+  int trackScreenScroll = 0;
+
   if (ses->trackScreenCursor) {
 #ifdef ENABLE_SPEECH_SUPPORT
     if (!spk.track.isActive)
@@ -638,25 +699,33 @@ doUpdate (void) {
       }
 
       /* If the cursor moves in cursor tracking mode: */
-      if (!isRouting() && (scr.posx != ses->trkx || scr.posy != ses->trky)) {
-        int oldx = ses->winx;
-        int oldy = ses->winy;
+      if (!isRouting()) {
+        if ((scr.posx != ses->trkx) || (scr.posy != ses->trky)) {
+          int oldx = ses->winx;
+          int oldy = ses->winy;
 
-        trackScreenCursor(0);
-        logMessage(LOG_CATEGORY(CURSOR_TRACKING),
-                   "scr=%u csr=[%u,%u]->[%u,%u] win=[%u,%u]->[%u,%u]",
-                   scr.number,
-                   ses->trkx, ses->trky, scr.posx, scr.posy,
-                   oldx, oldy, ses->winx, ses->winy);
+          trackScreenCursor(0);
+          logMessage(LOG_CATEGORY(CURSOR_TRACKING),
+                     "scr=%u csr=[%u,%u]->[%u,%u] win=[%u,%u]->[%u,%u]",
+                     scr.number,
+                     ses->trkx, ses->trky, scr.posx, scr.posy,
+                     oldx, oldy, ses->winx, ses->winy);
 
-        ses->spkx = ses->trkx = scr.posx;
-        ses->spky = ses->trky = scr.posy;
-      } else if (checkScreenPointer()) {
-        screenPointerMoved = 1;
+          ses->spkx = ses->trkx = scr.posx;
+          ses->spky = ses->trky = scr.posy;
+        } else if (checkScreenPointer()) {
+          screenPointerHasMoved = 1;
+        } else {
+          trackScreenScroll = 1;
+        }
       }
     }
+  } else {
+    trackScreenScroll = 1;
   }
 
+  checkScreenScroll(trackScreenScroll);
+
 #ifdef ENABLE_SPEECH_SUPPORT
   if (spk.canAutospeak) {
     int isAutospeaking = isAutospeakActive();
@@ -673,7 +742,7 @@ doUpdate (void) {
 
   /* There are a few things to take care of if the display has moved. */
   if ((ses->winx != oldwinx) || (ses->winy != oldwiny)) {
-    if (!screenPointerMoved) highlightBrailleWindowLocation();
+    if (!screenPointerHasMoved) highlightBrailleWindowLocation();
 
     /* Attributes are blinking.
      * We could check to see if we changed screen, but that doesn't
@@ -819,9 +888,15 @@ doUpdate (void) {
 #endif /* ENABLE_CONTRACTED_BRAILLE */
       {
         int windowColumns = MIN(textCount, scr.cols-ses->winx);
-        ScreenCharacter characters[textLength];
 
+        ScreenCharacter characters[textLength];
         readScreen(ses->winx, ses->winy, windowColumns, brl.textRows, characters);
+
+        if (prefs.wordWrap) {
+          int columns = getWordWrapLength(ses->winy, ses->winx, windowColumns);
+          if (columns < windowColumns) windowColumns = columns;
+        }
+
         if (windowColumns < textCount) {
           /* We got a rectangular piece of text with readScreen but the display
            * is in an off-right position with some cells at the end blank
@@ -1015,7 +1090,7 @@ ASYNC_ALARM_CALLBACK(handleUpdateAlarm) {
 static void
 setUpdateAlarm (void) {
   if (!updateSuspendCount && !updateAlarm) {
-    asyncSetAlarmTo(&updateAlarm, &updateTime, handleUpdateAlarm, NULL);
+    asyncNewAbsoluteAlarm(&updateAlarm, &updateTime, handleUpdateAlarm, NULL);
   }
 }
 
