@@ -2,14 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2017 by The BRLTTY Developers.
+ * Copyright (C) 1995-2018 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
  * Web Page: http://brltty.com/
  *
@@ -1119,8 +1119,8 @@ usbSchedulePendingInputRequest (UsbEndpoint *endpoint) {
     if (!*delay) *delay = 1;
     *delay = MIN(*delay, USB_INPUT_INTERRUPT_DELAY_MAXIMUM);
 
-    asyncSetAlarmIn(&endpoint->direction.input.pending.alarm, *delay,
-                    usbHandleSchedulePendingInputRequest, endpoint);
+    asyncNewRelativeAlarm(&endpoint->direction.input.pending.alarm, *delay,
+                          usbHandleSchedulePendingInputRequest, endpoint);
 
     *delay += 1;
   }
@@ -1640,6 +1640,6 @@ usbCloseChannel (UsbChannel *channel) {
 }
 
 int
-isUsbDevice (const char **identifier) {
-  return isQualifiedDevice(identifier, "usb");
+isUsbDeviceIdentifier (const char **identifier) {
+  return hasQualifier(identifier, "usb");
 }
